/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.kommentar;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.util.function.Predicate;
import java.util.stream.Stream;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.server.RepresentationModelAssembler;
import org.springframework.stereotype.Component;

import de.ozgcloud.alfa.common.CollectionModelBuilder;
import de.ozgcloud.alfa.common.ModelBuilder;
import de.ozgcloud.alfa.common.binaryfile.BinaryFileController;
import de.ozgcloud.alfa.kommentar.KommentarCommandController.KommentarCommandByVorgangController;
import de.ozgcloud.alfa.vorgang.Vorgang.VorgangStatus;
import de.ozgcloud.alfa.vorgang.VorgangController;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;

@Component
class KommentarModelAssembler implements RepresentationModelAssembler<Kommentar, EntityModel<Kommentar>> {

	static final String REL_CREATE = "create-kommentar";
	static final String REL_EDIT = "edit";
	static final String REL_ATTACHMENTS = "attachments";
	static final String REL_UPLOAD_FILE = "uploadFile";

	private static final Predicate<Kommentar> HAS_ATTACHMENTS = kommentar -> !kommentar.getAttachments().isEmpty();

	@Autowired
	private VorgangController vorgangController;

	@Override
	public EntityModel<Kommentar> toModel(Kommentar kommentar) {
		var vorgang = vorgangController.getVorgang(kommentar.getVorgangId());
		return buildModel(kommentar, vorgang);
	}

	EntityModel<Kommentar> buildModel(Kommentar kommentar, VorgangWithEingang vorgang) {
		var selfLink = linkTo(KommentarController.class).slash(kommentar.getId());
		var commandLink = linkTo(methodOn(KommentarCommandController.class).editKommentar(null, kommentar.getId(), kommentar.getVersion()));
		var attachmentsLink = linkTo(methodOn(KommentarController.class).getAttachments(kommentar.getId()));

		return ModelBuilder.fromEntity(kommentar).addLink(selfLink.withSelfRel())
				.ifMatch(() -> isEditable(vorgang))
				.addLink(commandLink.withRel(REL_EDIT))
				.ifMatch(HAS_ATTACHMENTS)
				.addLink(attachmentsLink.withRel(REL_ATTACHMENTS))
				.buildModel();
	}

	public CollectionModel<EntityModel<Kommentar>> toCollectionModel(Stream<Kommentar> entities, String vorgangId) {
		var vorgang = vorgangController.getVorgang(vorgangId);

		return CollectionModelBuilder.fromEntities(entities.map(kommentar -> buildModel(kommentar, vorgang)))
				.addLink(linkTo(KommentarController.class).withSelfRel())
				.addLink(linkTo(BinaryFileController.class).slash(vorgangId).slash("kommentarAttachment").slash("file").withRel(REL_UPLOAD_FILE))
				.ifMatch(kommentareModel -> isEditable(vorgang))
				.addLink(linkTo(methodOn(KommentarCommandByVorgangController.class).createKommentar(null, vorgangId))
						.withRel(REL_CREATE))
				.buildModel();
	}

	private boolean isEditable(VorgangWithEingang vorgang) {
		return vorgang.getStatus() != VorgangStatus.ZU_LOESCHEN;
	}
}