package de.ozgcloud.alfa.loeschanforderung;

import java.util.List;
import java.util.Optional;

import jakarta.validation.Valid;

import org.apache.commons.collections.MapUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import de.itvsh.kop.common.errorhandling.TechnicalException;
import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItem;
import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItemService;
import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.common.command.CommandService;
import de.ozgcloud.alfa.common.command.CreateCommand;
import de.ozgcloud.alfa.common.command.RequiredOrder;
import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserRole;
import de.ozgcloud.alfa.vorgang.Vorgang;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Validated
@Service
class LoeschAnforderungService {

	@Autowired
	private LoeschAnforderungMapper mapper;

	@Autowired
	private CommandService commandService;

	@Autowired
	private VorgangAttachedItemService vorgangAttachedItemService;

	@Autowired
	private CurrentUserService currentUserService;

	public Command createLoeschAnforderung(@Valid @RequiredOrder(CommandOrder.VORGANG_ZUM_LOESCHEN_MARKIEREN) CreateCommand command, Vorgang vorgang,
			long vorgangVersion) {
		createVorgangAttachedItem(vorgang);

		return createCreateCommand(command, vorgang, vorgangVersion);
	}

	void createVorgangAttachedItem(Vorgang vorgang) {
		vorgangAttachedItemService.createNewLoeschAnforderung(buildLoeschAnforderung(vorgang), vorgang.getId());
	}

	private LoeschAnforderung buildLoeschAnforderung(Vorgang vorgang) {
		return LoeschAnforderung.builder()
				.requestedBy(currentUserService.getUserId())
				.prevStatus(vorgang.getStatus())
				.build();
	}

	Command createCreateCommand(CreateCommand command, Vorgang vorgang, long vorgangVersion) {
		var createCommand = command.toBuilder().relationId(vorgang.getId()).vorgangId(vorgang.getId()).build();

		return commandService.createCommand(createCommand, vorgangVersion);
	}

	public Command vorgangLoeschen(CreateCommand command, String loeschAnforderungId) {
		var loeschAnforderung = getById(loeschAnforderungId);

		command = command.toBuilder().vorgangId(loeschAnforderung.getVorgangId()).relationId(loeschAnforderung.getVorgangId()).build();

		return commandService.createCommand(command);
	}

	public LoeschAnforderung getById(String id) {
		var vorgangAttachedItem = vorgangAttachedItemService.getById(id);

		return mapper.fromVorgangAttachedItem(vorgangAttachedItem);
	}

	public Command zuruecknehmen(VorgangAttachedItem vorgangAttachedItem, Vorgang vorgang) {
		var deleteAttachedItemCommand = vorgangAttachedItemService.deleteAttachedItem(vorgangAttachedItem);

		var loeschAnforderng = mapper.fromVorgangAttachedItem(vorgangAttachedItem);
		var changeStatusCommand = setPreviousVorgangStatus(vorgang, loeschAnforderng);

		return commandService.createCommand(
				buildDeleteLoeschAnforderungCommand(deleteAttachedItemCommand, changeStatusCommand, vorgang, loeschAnforderng),
				vorgang.getVersion());
	}

	public boolean isAllowedToVorgangLoeschen(LoeschAnforderung loeschAnforderung) {
		return hasRequiredRoles() && isDifferentUser(loeschAnforderung);
	}

	private boolean hasRequiredRoles() {
		return currentUserService.hasRole(UserRole.VERWALTUNG_LOESCHEN);
	}

	private boolean isDifferentUser(LoeschAnforderung loeschAnforderung) {
		return !currentUserService.getUserId().equals(loeschAnforderung.getRequestedBy());
	}

	Command setPreviousVorgangStatus(Vorgang vorgang, LoeschAnforderung loeschAnforderung) {
		var createCommand = createSetPreviousVorgangStatusCreateCommand(vorgang, loeschAnforderung);

		return commandService.createCommand(createCommand, vorgang.getVersion());
	}

	CreateCommand createSetPreviousVorgangStatusCreateCommand(Vorgang vorgang, LoeschAnforderung loeschAnforderung) {
		return CreateCommand.builder()
				.order(getCommandOrderForPreviousStatus(loeschAnforderung))
				.vorgangId(loeschAnforderung.getVorgangId())
				.relationId(loeschAnforderung.getVorgangId())
				.relationVersion(vorgang.getVersion()).build();
	}

	CommandOrder getCommandOrderForPreviousStatus(LoeschAnforderung loeschAnforderung) {
		return switch (loeschAnforderung.getPrevStatus()) {
			case ABGESCHLOSSEN -> CommandOrder.VORGANG_ABSCHLIESSEN;
			case VERWORFEN -> CommandOrder.VORGANG_VERWERFEN;
			default -> throw new TechnicalException(
					String.format("Unexpected prevStatus %s in LoeschAnforderung.", loeschAnforderung.getPrevStatus()));
		};
	}

	CreateCommand buildDeleteLoeschAnforderungCommand(Command deleteAttachedItemCommand, Command changeStatusCommand, Vorgang vorgang,
			LoeschAnforderung loeschAnforderung) {
		return CreateCommand.builder()
				.order(CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN)
				.vorgangId(vorgang.getId())
				.relationId(loeschAnforderung.getId())
				.body(buildDeleteLoeschAnforderung(changeStatusCommand.getId(), deleteAttachedItemCommand.getId()))
				.build();
	}

	DeleteLoeschAnforderung buildDeleteLoeschAnforderung(String changeStatusCommandId, String deleteVorgangAttachedItemCommandId) {
		return DeleteLoeschAnforderung.builder()
				.changeStatusCommandId(changeStatusCommandId)
				.deleteAttachedItemCommandId(deleteVorgangAttachedItemCommandId)
				.build();
	}

	public void deleteLoeschAnforderung(String loeschAnforderungId) {
		var vorgangAttachedItem = vorgangAttachedItemService.getById(loeschAnforderungId);

		vorgangAttachedItemService.deleteAttachedItem(vorgangAttachedItem);
	}

	public Optional<VorgangAttachedItem> findLoeschAnforderung(String vorgangId) {
		var vorgangAttachedItems = vorgangAttachedItemService.findLoeschAnforderung(vorgangId).toList();

		if (vorgangAttachedItems.isEmpty()) {
			return Optional.empty();
		}
		if (vorgangAttachedItems.size() > 1) {
			deleteDuplicateLoeschAnforderung(vorgangAttachedItems, vorgangId);
		}
		return Optional.of(vorgangAttachedItems.get(0));
	}

	void deleteDuplicateLoeschAnforderung(List<VorgangAttachedItem> items, String vorgangId) {
		LOG.warn("Found {} Löschanforderungen for vorgang {}, only one was expected.", items.size(), vorgangId);
		items.subList(1, items.size()).forEach(vorgangAttachedItemService::deleteAttachedItem);
	}

	public void reCreateLoeschAnforderung(Command command) {
		var deleteLoschAnforderungCommand = commandService.getById(getDeleteVorgangAttachedItemCommandId(command));
		var loeschAnforderung = mapper.fromCommand(deleteLoschAnforderungCommand);

		vorgangAttachedItemService.createNewLoeschAnforderung(loeschAnforderung, command.getVorgangId());
	}

	private String getDeleteVorgangAttachedItemCommandId(Command command) {
		return MapUtils.getString(command.getBody(), DeleteLoeschAnforderung.DELETE_ATTACHED_ITEM_COMMAND_ID_FIELD);
	}

	public String getChangeStatusCommandId(Command command) {
		return MapUtils.getString(command.getBody(), DeleteLoeschAnforderung.CHANGE_STATUS_COMMAND_ID_FIELD);
	}
}