package de.ozgcloud.alfa.loeschanforderung;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.util.List;
import java.util.Objects;
import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.Link;
import org.springframework.hateoas.LinkRelation;
import org.springframework.hateoas.server.RepresentationModelProcessor;
import org.springframework.stereotype.Component;

import de.ozgcloud.alfa.common.ModelBuilder;
import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItem;
import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserRole;
import de.ozgcloud.alfa.loeschanforderung.LoeschAnforderungController.LoeschAnforderungByVorgangController;
import de.ozgcloud.alfa.vorgang.Vorgang;
import de.ozgcloud.alfa.vorgang.Vorgang.VorgangStatus;

@Component
class LoeschAnforderungVorgangProzessor implements RepresentationModelProcessor<EntityModel<? extends Vorgang>> {

	static final LinkRelation REL_LOESCHEN_ANFORDERN = LinkRelation.of("loeschen_anfordern");
	static final LinkRelation REL_LOESCH_ANFORDERUNG = LinkRelation.of("loesch_anforderung");

	static final List<String> USER_ROLES_ALLOWED_TO_LOESCHEN_ANFORDERN = List.of(UserRole.VERWALTUNG_USER, UserRole.EINHEITLICHER_ANSPRECHPARTNER);
	static final List<VorgangStatus> VORGANG_STATUSES_ALLOWED_TO_LOESCHEN_ANFORDERN = List.of(VorgangStatus.VERWORFEN, VorgangStatus.ABGESCHLOSSEN);

	@Autowired
	private LoeschAnforderungService service;

	@Autowired
	private CurrentUserService currentUserService;

	@Override
	public EntityModel<? extends Vorgang> process(EntityModel<? extends Vorgang> model) {
		var vorgang = model.getContent();

		if (Objects.isNull(vorgang)) {
			return model;
		}

		var vorgangAttachedItem = service.findLoeschAnforderung(vorgang.getId());

		return ModelBuilder.fromModel(model)
				.ifMatch(this::isLoeschenAnfordernAllowed)
				.addLink(buildLoschenAnfordernLink(vorgang))
				.addLink(buildLoschAnforderungLink(vorgangAttachedItem))
				.buildModel();
	}

	boolean isLoeschenAnfordernAllowed(Vorgang vorgang) {
		return isAllowedStatus(vorgang) && isAllowedUserRole();
	}

	private boolean isAllowedStatus(Vorgang vorgang) {
		return VORGANG_STATUSES_ALLOWED_TO_LOESCHEN_ANFORDERN.contains(vorgang.getStatus());
	}

	private boolean isAllowedUserRole() {
		return USER_ROLES_ALLOWED_TO_LOESCHEN_ANFORDERN.stream().anyMatch(currentUserService::hasRole);
	}

	private Link buildLoschenAnfordernLink(Vorgang vorgang) {
		return linkTo(methodOn(LoeschAnforderungByVorgangController.class).createLoeschAnforderung(vorgang.getId(), vorgang.getVersion(), null))
				.withRel(REL_LOESCHEN_ANFORDERN);
	}

	private Optional<Link> buildLoschAnforderungLink(Optional<VorgangAttachedItem> vorgangAttachedItem) {
		return vorgangAttachedItem.map(item -> linkTo(methodOn(LoeschAnforderungController.class).getLoeschanforderung(item.getId()))
				.withRel(REL_LOESCH_ANFORDERUNG));
	}
}
