/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.binaryfile;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.util.concurrent.CompletableFuture;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.util.unit.DataSize;

import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;

@AutoConfigureMockMvc
@SpringBootTest
@WithMockUser
class BinaryFileITCase {

	@MockBean
	private BinaryFileRemoteService remoteService;

	@Autowired
	private MockMvc mockMvc;

	@WithMockUser
	@Nested
	class TestUploadFile {

		private final String SINGLE_PATH = BinaryFileController.PATH + "/{vorgangId}/{field}/file";
		private final CompletableFuture<FileId> future = CompletableFuture.completedFuture(BinaryFileTestFactory.FILE_ID);

		public static final byte[] DATA_OVER_3_MB = BinaryFileTestFactory.createByteOfSize(DataSize.ofMegabytes(41));
		public static final MockMultipartFile TEST_FILE = new MockMultipartFile("file", BinaryFileTestFactory.NAME,
				BinaryFileTestFactory.CONTENT_TYPE, DATA_OVER_3_MB);

		@BeforeEach
		void mockStub() {
			when(remoteService.uploadFile(any())).thenReturn(future);
		}

		@Nested
		class TestDefault {
			@Test
			void shouldReturnConstraintViolationException() throws Exception {
				var result = callEndpoint(TEST_FILE, BinaryFileTestFactory.FIELD);

				result.andExpect(status().isUnprocessableEntity());
			}

			@Test
			void shouldReturnConstraintViolationExceptionData() throws Exception {
				var result = callEndpoint(TEST_FILE, BinaryFileTestFactory.FIELD);

				result.andExpect(jsonPath("issues[0].field").value("uploadBinaryFileRequest"))
						.andExpect(jsonPath("issues[0].messageCode").value("validation_field_file_size_exceeded"))
						.andExpect(jsonPath("issues[0].message").value("validation_field_file_size_exceeded"))
						.andExpect(jsonPath("issues[0].parameters[0].name").value("unit"))
						.andExpect(jsonPath("issues[0].parameters[0].value").value(UploadBinaryFileSizeValidator.DEFAULT_UNIT_STRING));
			}

			@Test
			void shouldContainsExceptionDataOnDefaultMaxSize() throws Exception {
				var result = callEndpoint(TEST_FILE, BinaryFileTestFactory.FIELD);

				result.andExpect(jsonPath("issues[0].parameters[1].name").value("max"))
						.andExpect(jsonPath("issues[0].parameters[1].value").value("40"));
			}
		}

		@Nested
		class TestForPostfachNachricht {

			private final static String FIELD = "postfachNachrichtAttachment";

			@Test
			void shouldContainsExceptionDataOnPostfachNachrichtMaxSize() throws Exception {
				var result = callEndpoint(TEST_FILE, FIELD);

				result.andExpect(jsonPath("issues[0].parameters[1].name").value("max"))
						.andExpect(jsonPath("issues[0].parameters[1].value").value("3"));
			}
		}

		private ResultActions callEndpoint(MockMultipartFile file, String field) throws Exception {
			return mockMvc.perform(multipart(SINGLE_PATH, VorgangHeaderTestFactory.ID, field).file(file).with(csrf()));
		}
	}
}