package de.ozgcloud.alfa.loeschanforderung;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.UUID;
import java.util.stream.Stream;

import org.apache.commons.lang3.StringUtils;
import org.assertj.core.api.InstanceOfAssertFactories;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.itvsh.kop.common.errorhandling.TechnicalException;
import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItem;
import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItemService;
import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItemTestFactory;
import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.common.command.CommandService;
import de.ozgcloud.alfa.common.command.CommandTestFactory;
import de.ozgcloud.alfa.common.command.CreateCommand;
import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserId;
import de.ozgcloud.alfa.common.user.UserProfileTestFactory;
import de.ozgcloud.alfa.common.user.UserRole;
import de.ozgcloud.alfa.vorgang.Vorgang;
import de.ozgcloud.alfa.vorgang.Vorgang.VorgangStatus;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import de.ozgcloud.alfa.vorgang.VorgangWithEingangTestFactory;

class LoeschAnforderungServiceTest {

	@Spy
	@InjectMocks
	private LoeschAnforderungService service;
	@Mock
	private LoeschAnforderungMapper mapper;

	@Mock
	private CommandService commandService;

	@Mock
	private VorgangAttachedItemService vorgangAttachedItemService;

	@Mock
	private CurrentUserService currentUserService;

	@Nested
	class TestCreateVorgangAttachedItem {

		@Captor
		private ArgumentCaptor<LoeschAnforderung> commandBodyArgumentCaptor;

		@BeforeEach
		void init() {
			when(currentUserService.getUserId()).thenReturn(UserProfileTestFactory.ID);
		}

		@Test
		void shouldCallCurrentUserService() {
			service.createLoeschAnforderung(CommandTestFactory.createCreateCommand(), VorgangWithEingangTestFactory.create(),
					VorgangHeaderTestFactory.VERSION);

			verify(currentUserService).getUserId();
		}

		@Test
		void shouldCallVorgangAttachedItemService() {
			service.createLoeschAnforderung(CommandTestFactory.createCreateCommand(), VorgangWithEingangTestFactory.create(),
					VorgangHeaderTestFactory.VERSION);

			verify(vorgangAttachedItemService).createNewLoeschAnforderung(commandBodyArgumentCaptor.capture(), eq(VorgangHeaderTestFactory.ID));

			assertThat(commandBodyArgumentCaptor.getValue()).asInstanceOf(InstanceOfAssertFactories.type(LoeschAnforderung.class))
					.extracting(LoeschAnforderung::getPrevStatus, LoeschAnforderung::getRequestedBy)
					.containsExactly(VorgangHeaderTestFactory.STATUS, UserProfileTestFactory.ID);
		}
	}

	@Nested
	class TestCreateCreateCommand {

		@Captor
		private ArgumentCaptor<CreateCommand> createCommandArgumentCaptor;

		@Test
		void shouldCallCommandService() {
			var createCommand = CreateCommand.builder().order(CommandOrder.VORGANG_ZUM_LOESCHEN_MARKIEREN).build();
			service.createCreateCommand(createCommand, VorgangWithEingangTestFactory.create(), VorgangHeaderTestFactory.VERSION);

			verify(commandService).createCommand(createCommandArgumentCaptor.capture(), anyLong());

			assertThat(createCommandArgumentCaptor.getValue().getOrder()).isEqualTo(CommandOrder.VORGANG_ZUM_LOESCHEN_MARKIEREN);
			assertThat(createCommandArgumentCaptor.getValue().getVorgangId()).isEqualTo(VorgangHeaderTestFactory.ID);
			assertThat(createCommandArgumentCaptor.getValue().getRelationId()).isEqualTo(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldReturnCommand() {
			var command = CommandTestFactory.create();
			when(commandService.createCommand(any(), anyLong())).thenReturn(command);

			var result = service.createCreateCommand(
					CommandTestFactory.createCreateCommand(),
					VorgangWithEingangTestFactory.create(),
					VorgangHeaderTestFactory.VERSION);

			assertThat(result).isEqualTo(command);
		}
	}

	@Nested
	class TestCreateLoeschAnforderung {

		private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();
		private final CreateCommand createCommand = CommandTestFactory.createCreateCommand();

		@BeforeEach
		void init() {
			doNothing().when(service).createVorgangAttachedItem(vorgang);
		}

		@Test
		void shouldCallCreateVorgangAttachedItem() {
			service.createLoeschAnforderung(CommandTestFactory.createCreateCommand(), vorgang, CommandTestFactory.RELATION_VERSION);

			verify(service).createVorgangAttachedItem(vorgang);
		}

		@Test
		void shouldCreateCreateCommand() {
			service.createLoeschAnforderung(createCommand, vorgang, VorgangHeaderTestFactory.VERSION);

			verify(service).createCreateCommand(createCommand, vorgang, VorgangHeaderTestFactory.VERSION);
		}
	}

	@Nested
	class TestIsAllowedToVorgangLoeschen {

		@Test
		void shouldReturnTrue() {
			when(currentUserService.getUserId()).thenReturn(UserId.from(UUID.randomUUID()));
			when(currentUserService.hasRole(UserRole.VERWALTUNG_LOESCHEN)).thenReturn(true);

			var isAllowed = service.isAllowedToVorgangLoeschen(LoeschAnforderungTestFactory.create());

			assertThat(isAllowed).isTrue();
		}

		@Test
		void shouldReturnFalseIfRolesMissing() {
			when(currentUserService.hasRole(UserRole.VERWALTUNG_LOESCHEN)).thenReturn(false);

			var isAllowed = service.isAllowedToVorgangLoeschen(LoeschAnforderungTestFactory.create());

			assertThat(isAllowed).isFalse();
		}

		@Test
		void shouldReturnFalseIfRequestorIsSameUser() {
			when(currentUserService.getUserId()).thenReturn(UserId.from(LoeschAnforderungTestFactory.REQUESTED_BY.toString()));
			when(currentUserService.hasRole(UserRole.VERWALTUNG_LOESCHEN)).thenReturn(true);

			var isAllowed = service.isAllowedToVorgangLoeschen(LoeschAnforderungTestFactory.create());

			assertThat(isAllowed).isFalse();
		}
	}

	@Nested
	@DisplayName("Get LoeschAnforderung resouce by id")
	class TestGetById {

		private final VorgangAttachedItem vorgangAttachedItem = VorgangAttachedItemTestFactory.create();
		private final LoeschAnforderung loeschAnforderung = LoeschAnforderungTestFactory.create();

		@BeforeEach
		void init() {
			when(vorgangAttachedItemService.getById(LoeschAnforderungTestFactory.ID)).thenReturn(vorgangAttachedItem);
			when(mapper.fromVorgangAttachedItem(any())).thenReturn(loeschAnforderung);
		}

		@Test
		void shouldCallVorgangAttachedItemService() {
			service.getById(LoeschAnforderungTestFactory.ID);

			verify(vorgangAttachedItemService).getById(LoeschAnforderungTestFactory.ID);
		}

		@Test
		void shouldMapVorgangAttachedItem() {
			service.getById(LoeschAnforderungTestFactory.ID);

			verify(mapper).fromVorgangAttachedItem(vorgangAttachedItem);
		}

		@Test
		void shouldReturnLoeschAnforderung() {
			var result = service.getById(LoeschAnforderungTestFactory.ID);

			assertThat(result).isEqualTo(loeschAnforderung);
		}
	}

	@Nested
	@DisplayName("Vorgang loeschen")
	class TestVorgangLoeschen {

		private LoeschAnforderung loeschAnforderung = LoeschAnforderungTestFactory.create();
		private Command created = CommandTestFactory.create();

		@Captor
		private ArgumentCaptor<CreateCommand> createCommandArgumentCaptor;

		@BeforeEach
		void init() {
			doReturn(loeschAnforderung).when(service).getById(LoeschAnforderungTestFactory.ID);
			when(commandService.createCommand(any())).thenReturn(created);
		}

		@Test
		void shouldCallCommandService() {
			service.vorgangLoeschen(CommandTestFactory.createCreateCommand(), LoeschAnforderungTestFactory.ID);

			verify(commandService).createCommand(createCommandArgumentCaptor.capture());
			assertThat(createCommandArgumentCaptor.getValue()).usingRecursiveComparison()
					.ignoringFields("vorgangId", "relationId").isEqualTo(CommandTestFactory.createCreateCommand());
			assertThat(createCommandArgumentCaptor.getValue().getVorgangId()).isEqualTo(VorgangHeaderTestFactory.ID);
			assertThat(createCommandArgumentCaptor.getValue().getRelationId()).isEqualTo(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldReturnCreatedCommand() {
			var result = service.vorgangLoeschen(
					CreateCommand.builder().order(CommandOrder.VORGANG_LOESCHEN).build(), LoeschAnforderungTestFactory.ID);

			assertThat(result).isEqualTo(created);
		}
	}

	@Nested
	class TestLoeschAnforderungZuruecknehmen {

		private Vorgang vorgang = VorgangHeaderTestFactory.create();
		private VorgangAttachedItem vorgangAttachedItem = VorgangAttachedItemTestFactory.createBuilder().item(LoeschAnforderungTestFactory.asMap())
				.build();
		private LoeschAnforderung loeschAnforderung;
		private Command changeStatusCommand;
		private Command deleteAttachedItemCommand;
		private Command deleteLoeschAnforderungCommand;
		private CreateCommand deleteLoeschAnforderungCreateCommand;

		@BeforeEach
		void init() {
			prepareLoeschAnforderung();
			prepareDeleteAttachedItemCommand();
			prepareChangeVorgangStatusCommand();
			prepareDeleteLoeschAnforderungCommand();
			prepareResultCommand();
		}

		void prepareLoeschAnforderung() {
			loeschAnforderung = LoeschAnforderungTestFactory.create();
			when(mapper.fromVorgangAttachedItem(any())).thenReturn(loeschAnforderung);
		}

		void prepareDeleteAttachedItemCommand() {
			deleteAttachedItemCommand = CommandTestFactory.create();
			when(vorgangAttachedItemService.deleteAttachedItem(vorgangAttachedItem)).thenReturn(deleteAttachedItemCommand);
		}

		void prepareChangeVorgangStatusCommand() {
			changeStatusCommand = CommandTestFactory.create();
			doReturn(changeStatusCommand).when(service).setPreviousVorgangStatus(vorgang, loeschAnforderung);
		}

		void prepareDeleteLoeschAnforderungCommand() {
			deleteLoeschAnforderungCreateCommand = CommandTestFactory.createCreateCommand();
			doReturn(deleteLoeschAnforderungCreateCommand).when(service)
					.buildDeleteLoeschAnforderungCommand(deleteAttachedItemCommand, changeStatusCommand, vorgang, loeschAnforderung);
		}

		void prepareResultCommand() {
			deleteLoeschAnforderungCommand = CommandTestFactory.create();
			when(commandService.createCommand(deleteLoeschAnforderungCreateCommand, VorgangHeaderTestFactory.VERSION)).thenReturn(
					deleteLoeschAnforderungCommand);
		}

		@Test
		void shouldMapVorgangAttachedItemToLoeschAnforderung() {
			service.zuruecknehmen(vorgangAttachedItem, vorgang);

			verify(mapper).fromVorgangAttachedItem(vorgangAttachedItem);
		}

		@Test
		void shouldDeleteVorgangAttacheditem() {
			service.zuruecknehmen(vorgangAttachedItem, vorgang);

			verify(vorgangAttachedItemService).deleteAttachedItem(vorgangAttachedItem);
		}

		@Test
		void shouldSetPreviousVorgangStatus() {
			service.zuruecknehmen(vorgangAttachedItem, vorgang);

			verify(service).setPreviousVorgangStatus(vorgang, loeschAnforderung);
		}

		@Test
		void shouldCreateDeleteLoeschAnforderungCommand() {
			service.zuruecknehmen(vorgangAttachedItem, vorgang);

			verify(service).buildDeleteLoeschAnforderungCommand(deleteAttachedItemCommand, changeStatusCommand, vorgang, loeschAnforderung);
		}

		@Test
		void shouldReturnCommand() {
			var resultCommand = service.zuruecknehmen(vorgangAttachedItem, vorgang);

			assertThat(resultCommand).isEqualTo(deleteLoeschAnforderungCommand);
		}
	}

	@Nested
	class TestCreateDeleteLoeschAnforderungCommand {
		private Command deleteAttachedItemCommand;
		private Command changeVorgangStatusCommand;
		private Vorgang vorgang;
		private LoeschAnforderung loeschAnforderung;

		@BeforeEach
		void init() {
			deleteAttachedItemCommand = CommandTestFactory.create();
			changeVorgangStatusCommand = CommandTestFactory.create();
			vorgang = VorgangHeaderTestFactory.create();
			loeschAnforderung = LoeschAnforderungTestFactory.create();
		}

		@Test
		void shouldHaveOrder() {
			var createCommand = service.buildDeleteLoeschAnforderungCommand(deleteAttachedItemCommand, changeVorgangStatusCommand, vorgang,
					loeschAnforderung);

			assertThat(createCommand.getOrder()).isEqualTo(CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN);
		}

		@Test
		void shouldContainsDeleteAttachedItemCommandId() {
			var createCommand = service.buildDeleteLoeschAnforderungCommand(deleteAttachedItemCommand, changeVorgangStatusCommand, vorgang,
					loeschAnforderung);

			assertThat(createCommand.getBody()).asInstanceOf(InstanceOfAssertFactories.type(DeleteLoeschAnforderung.class))
					.extracting(DeleteLoeschAnforderung::getDeleteAttachedItemCommandId).isEqualTo(deleteAttachedItemCommand.getId());
		}

		@Test
		void shouldContainsChangeVorgangStatusCommandId() {
			var createCommand = service.buildDeleteLoeschAnforderungCommand(deleteAttachedItemCommand, changeVorgangStatusCommand, vorgang,
					loeschAnforderung);

			assertThat(createCommand.getBody()).asInstanceOf(InstanceOfAssertFactories.type(DeleteLoeschAnforderung.class))
					.extracting(DeleteLoeschAnforderung::getChangeStatusCommandId).isEqualTo(changeVorgangStatusCommand.getId());
		}

		@Test
		void shouldHaveVorgangId() {
			var createCommand = service.buildDeleteLoeschAnforderungCommand(deleteAttachedItemCommand, changeVorgangStatusCommand, vorgang,
					loeschAnforderung);

			assertThat(createCommand.getVorgangId()).isEqualTo(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldHaveRelationId() {
			var createCommand = service.buildDeleteLoeschAnforderungCommand(deleteAttachedItemCommand, changeVorgangStatusCommand, vorgang,
					loeschAnforderung);

			assertThat(createCommand.getRelationId()).isEqualTo(LoeschAnforderungTestFactory.ID);
		}
	}

	@Nested
	class TestGetCommandOrderForPreviousStatus {

		@Test
		void shouldReturnVorgangVerwerfenOrder() {
			var loeschAnforderung = LoeschAnforderungTestFactory.createBuilder().prevStatus(VorgangStatus.VERWORFEN).build();

			var order = service.getCommandOrderForPreviousStatus(loeschAnforderung);

			assertThat(order).isEqualTo(CommandOrder.VORGANG_VERWERFEN);
		}

		@Test
		void shouldReturnVorgangAbschliessenOrder() {
			var loeschAnforderung = LoeschAnforderungTestFactory.createBuilder().prevStatus(VorgangStatus.ABGESCHLOSSEN).build();

			var order = service.getCommandOrderForPreviousStatus(loeschAnforderung);

			assertThat(order).isEqualTo(CommandOrder.VORGANG_ABSCHLIESSEN);
		}

		@Test
		void shouldThrowTechnicalException() {
			var loeschAnforderung = LoeschAnforderungTestFactory.createBuilder().prevStatus(VorgangStatus.NEU).build();

			assertThatThrownBy(() -> service.getCommandOrderForPreviousStatus(loeschAnforderung)).isInstanceOf(
					TechnicalException.class);

		}
	}

	@Nested
	class TestCreateSetPreviousVorgangStatusCreateCommand {

		private LoeschAnforderung loeschAnforderung;
		private Vorgang vorgang;

		@BeforeEach
		void init() {
			loeschAnforderung = LoeschAnforderungTestFactory.createBuilder().prevStatus(VorgangStatus.ABGESCHLOSSEN).build();
			vorgang = VorgangHeaderTestFactory.create();
		}

		@Test
		void shouldSetOrder() {
			var command = service.createSetPreviousVorgangStatusCreateCommand(vorgang, loeschAnforderung);

			assertThat(command.getOrder()).isEqualTo(CommandOrder.VORGANG_ABSCHLIESSEN);
		}

		@Test
		void shouldSetVorgangId() {
			var command = service.createSetPreviousVorgangStatusCreateCommand(vorgang, loeschAnforderung);

			assertThat(command.getVorgangId()).isEqualTo(LoeschAnforderungTestFactory.VORGANG_ID);
		}

		@Test
		void shouldSetRelationId() {
			var command = service.createSetPreviousVorgangStatusCreateCommand(vorgang, loeschAnforderung);

			assertThat(command.getRelationId()).isEqualTo(LoeschAnforderungTestFactory.VORGANG_ID);
		}

		@Test
		void shouldSetVersion() {
			var command = service.createSetPreviousVorgangStatusCreateCommand(vorgang, loeschAnforderung);

			assertThat(command.getRelationVersion()).isEqualTo(VorgangHeaderTestFactory.VERSION);
		}

		@Test
		void shouldCallGetCommandOrderForPreviousStatus() {
			service.createSetPreviousVorgangStatusCreateCommand(vorgang, loeschAnforderung);

			verify(service).getCommandOrderForPreviousStatus(loeschAnforderung);
		}
	}

	@Nested
	class TestSetPreviousVorgangStatus {

		private LoeschAnforderung loeschAnforderung;
		private Vorgang vorgang;
		private CreateCommand createCommand;
		private Command command;

		@BeforeEach
		void init() {
			loeschAnforderung = LoeschAnforderungTestFactory.createBuilder().prevStatus(VorgangStatus.ABGESCHLOSSEN).build();
			vorgang = VorgangHeaderTestFactory.create();
			createCommand = CommandTestFactory.createCreateCommand();
			command = CommandTestFactory.create();

			doReturn(createCommand).when(service).createSetPreviousVorgangStatusCreateCommand(vorgang, loeschAnforderung);
			when(commandService.createCommand(createCommand, VorgangHeaderTestFactory.VERSION)).thenReturn(command);
		}

		@Test
		void shouldCreateCommand() {
			service.setPreviousVorgangStatus(vorgang, loeschAnforderung);

			verify(service).createSetPreviousVorgangStatusCreateCommand(vorgang, loeschAnforderung);
		}

		@Test
		void shouldCallCommandService() {
			service.setPreviousVorgangStatus(vorgang, loeschAnforderung);

			verify(commandService).createCommand(createCommand, VorgangHeaderTestFactory.VERSION);
		}

		@Test
		void shouldReturnCommand() {
			var result = service.setPreviousVorgangStatus(vorgang, loeschAnforderung);

			assertThat(result).isEqualTo(command);
		}
	}

	@Nested
	@DisplayName("FindVorgangAttachedItem")
	class TestFindVorgangAttachedItem {

		private final VorgangAttachedItem vorgangAttachedItem = VorgangAttachedItemTestFactory.create();
		private final Stream<VorgangAttachedItem> vorgangAttachedItems = Stream.of(vorgangAttachedItem);

		@DisplayName("on single result")
		@Nested
		class TestOnSingleLoeschAnforderung {

			@BeforeEach
			void init() {
				when(vorgangAttachedItemService.findLoeschAnforderung(VorgangHeaderTestFactory.ID)).thenReturn(vorgangAttachedItems);
			}

			@Test
			void shouldCallRemoteService() {
				service.findLoeschAnforderung(VorgangHeaderTestFactory.ID);

				verify(vorgangAttachedItemService).findLoeschAnforderung(VorgangHeaderTestFactory.ID);
			}

			@Test
			void shouldReturnResult() {
				var result = service.findLoeschAnforderung(VorgangHeaderTestFactory.ID);

				assertThat(result).isPresent().hasValue(vorgangAttachedItem);
			}
		}

		@DisplayName("on multiple result")
		@Nested
		class TestOnMultipleLoeschAnforderungen {

			private final Stream<VorgangAttachedItem> loeschAnforderungen = Stream.of(vorgangAttachedItem, vorgangAttachedItem);

			@BeforeEach
			void initMock() {
				when(vorgangAttachedItemService.findLoeschAnforderung(VorgangHeaderTestFactory.ID)).thenReturn(loeschAnforderungen);
			}

			@Test
			void shouldDeleteDuplicateLoeschAnforderung() {
				service.findLoeschAnforderung(VorgangHeaderTestFactory.ID);

				verify(service).deleteDuplicateLoeschAnforderung(any(), eq(VorgangHeaderTestFactory.ID));
			}

			@Test
			void shouldReturnFirstLoeschAnforderung() {
				var result = service.findLoeschAnforderung(VorgangHeaderTestFactory.ID);

				assertThat(result).isPresent().hasValue(vorgangAttachedItem);
			}
		}

		@DisplayName("delete duplicate loeschAnforderung")
		@Nested
		class TestDeleteDuplicateLoeschAnforderun {

			private final VorgangAttachedItem otherLoeschAnforderung = VorgangAttachedItemTestFactory.create();
			private final VorgangAttachedItem anotherLoeschAnforderung = VorgangAttachedItemTestFactory.create();
			private final List<VorgangAttachedItem> loeschAnforderungen = List.of(vorgangAttachedItem, otherLoeschAnforderung,
					anotherLoeschAnforderung);

			@Test
			void shouldDeleteAllExceptFirstElement() {
				service.deleteDuplicateLoeschAnforderung(loeschAnforderungen, VorgangHeaderTestFactory.ID);

				verify(vorgangAttachedItemService).deleteAttachedItem(otherLoeschAnforderung);
				verify(vorgangAttachedItemService).deleteAttachedItem(anotherLoeschAnforderung);
			}
		}
	}

	@DisplayName("Find Loeschanforderung")
	@Nested
	class TestFindLoeschAnforderung {

		private final VorgangAttachedItem vorgangAttachedItem = VorgangAttachedItemTestFactory.create();

		@BeforeEach
		void mock() {
			when(vorgangAttachedItemService.findLoeschAnforderung(any())).thenReturn(Stream.of(vorgangAttachedItem));
		}

		@Test
		void shouldCallService() {
			service.findLoeschAnforderung(VorgangHeaderTestFactory.ID);

			verify(vorgangAttachedItemService).findLoeschAnforderung(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldReturnResult() {
			var result = service.findLoeschAnforderung(VorgangHeaderTestFactory.ID);

			assertThat(result).isPresent().hasValue(vorgangAttachedItem);
		}
	}

	@DisplayName("Delete LoeschAnforderung")
	@Nested
	class TestDeleteLoeschAnforderung {

		private final VorgangAttachedItem vorgangAttachedItem = VorgangAttachedItemTestFactory.create();

		@BeforeEach
		void mock() {
			when(vorgangAttachedItemService.getById(anyString())).thenReturn(vorgangAttachedItem);
		}

		@Test
		void shouldGetLoeschAnforderung() {
			service.deleteLoeschAnforderung(LoeschAnforderungTestFactory.ID);

			verify(vorgangAttachedItemService).getById(LoeschAnforderungTestFactory.ID);
		}

		@Test
		void shouldDeleteVorgangAttachedItem() {
			service.deleteLoeschAnforderung(LoeschAnforderungTestFactory.ID);

			verify(vorgangAttachedItemService).deleteAttachedItem(vorgangAttachedItem);
		}
	}

	@DisplayName("Recreate LoeschAnforderung")
	@Nested
	class TestReCreateLoeschAnforderung {

		@DisplayName(StringUtils.EMPTY)
		@Nested
		class TestWorkflow {

			private final Command zuruecknehmenCommand = CommandTestFactory.createBuilder()
					.body(LoeschAnforderungTestFactory.createDeleteLoeschAnforderungBodyMap())
					.build();
			private final Command deleteVorgangAttachedItemCommand = CommandTestFactory.create();

			@BeforeEach
			void mock() {
				when(commandService.getById(anyString())).thenReturn(deleteVorgangAttachedItemCommand);
				when(mapper.fromCommand(any())).thenReturn(LoeschAnforderungTestFactory.create());
			}

			@Test
			void shouldGetLoeschAnforderungZuruecknehmenCommand() {
				service.reCreateLoeschAnforderung(zuruecknehmenCommand);

				verify(commandService).getById(LoeschAnforderungTestFactory.DELETE_VORGANG_ATTACHED_ITEM_ID);
			}

			@Test
			void shouldGetDeleteLoeschAnforderungCommand() {
				service.reCreateLoeschAnforderung(zuruecknehmenCommand);

				verify(commandService).getById(LoeschAnforderungTestFactory.DELETE_VORGANG_ATTACHED_ITEM_ID);
			}

			@Test
			void shouldCallMapper() {
				service.reCreateLoeschAnforderung(zuruecknehmenCommand);

				verify(mapper).fromCommand(deleteVorgangAttachedItemCommand);
			}

			@Test
			void shouldCreateNewLoeschAnforderung() {
				service.reCreateLoeschAnforderung(zuruecknehmenCommand);

				verify(vorgangAttachedItemService).createNewLoeschAnforderung(any(), eq(VorgangHeaderTestFactory.ID));
			}
		}
	}

	@DisplayName("Get changeStatus commandId")
	@Nested
	class TestGetChangedStatusCommandId {

		private final Command zuruecknehmenCommand = CommandTestFactory.createBuilder()
				.body(LoeschAnforderungTestFactory.createDeleteLoeschAnforderungBodyMap())
				.build();

		@Test
		void shouldReturnChangeStatusCommandId() {
			var commandId = service.getChangeStatusCommandId(zuruecknehmenCommand);

			assertThat(commandId).isEqualTo(LoeschAnforderungTestFactory.CHANGE_STATUS_COMMAND_ID);
		}
	}
}