package de.ozgcloud.alfa.loeschanforderung;

import static de.ozgcloud.alfa.common.UserProfileUrlProviderTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Optional;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.EnumSource;
import org.junit.jupiter.params.provider.EnumSource.Mode;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.Link;

import de.ozgcloud.alfa.common.UserProfileUrlProvider;
import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItemTestFactory;
import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserRole;
import de.ozgcloud.alfa.loeschanforderung.LoeschAnforderungController.LoeschAnforderungByVorgangController;
import de.ozgcloud.alfa.vorgang.Vorgang;
import de.ozgcloud.alfa.vorgang.Vorgang.VorgangStatus;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;

class LoeschAnforderungVorgangProzessorTest {

	@Spy
	@InjectMocks
	private LoeschAnforderungVorgangProzessor prozessor;

	@Mock
	private CurrentUserService currentUserService;

	@Mock
	private LoeschAnforderungService loeschAnforderungService;

	@DisplayName("Prozess")
	@Nested
	class TestProcess {

		@DisplayName("add links")
		@Nested
		class TestLinks {

			@BeforeEach
			void mock() {
				initUserProfileUrlProvider(new UserProfileUrlProvider());

				when(loeschAnforderungService.findLoeschAnforderung(VorgangHeaderTestFactory.ID)).thenReturn(
						Optional.of(VorgangAttachedItemTestFactory.create()));
			}

			@Test
			void shouldFindLoeschAnforderung() {
				prozessor.process(EntityModel.of(VorgangHeaderTestFactory.create()));

				verify(loeschAnforderungService).findLoeschAnforderung(VorgangHeaderTestFactory.ID);
			}

			@Test
			void shouldAddLoeschenAnfordernLink() {
				doReturn(true).when(prozessor).isLoeschenAnfordernAllowed(any());

				var model = prozessor.process(EntityModel.of(VorgangHeaderTestFactory.create()));

				assertThat(model.getLink(LoeschAnforderungVorgangProzessor.REL_LOESCHEN_ANFORDERN)).isPresent()
						.map(Link::getHref)
						.hasValue(LoeschAnforderungByVorgangController.BASE_PATH.replace("{vorgangId}", VorgangHeaderTestFactory.ID)
								.replace("{vorgangVersion}", String.valueOf(VorgangHeaderTestFactory.VERSION)));
			}

			@Test
			void shoulNotdAddLoeschenAnfordernLink() {
				doReturn(false).when(prozessor).isLoeschenAnfordernAllowed(any());

				var model = prozessor.process(EntityModel.of(VorgangHeaderTestFactory.create()));

				assertThat(model.getLink(LoeschAnforderungVorgangProzessor.REL_LOESCHEN_ANFORDERN)).isEmpty();
			}

			@Test
			void shouldAddLoeschAnforderungLink() {
				var vogang = VorgangHeaderTestFactory.createBuilder().status(Vorgang.VorgangStatus.ZU_LOESCHEN).build();

				var model = prozessor.process(EntityModel.of(vogang));

				var link = model.getLink(LoeschAnforderungVorgangProzessor.REL_LOESCH_ANFORDERUNG);
				assertThat(link).isPresent().map(Link::getHref)
						.hasValue(LoeschAnforderungController.BASE_PATH + "/" + VorgangAttachedItemTestFactory.ID);
			}

			@Test
			void shouldNotAddLoeschAnforderungLink() {
				when(loeschAnforderungService.findLoeschAnforderung(VorgangHeaderTestFactory.ID)).thenReturn(Optional.empty());
				var vorgang = VorgangHeaderTestFactory.createBuilder().status(Vorgang.VorgangStatus.ZU_LOESCHEN).build();

				var model = prozessor.process(EntityModel.of(vorgang));

				assertThat(model.getLink(LoeschAnforderungVorgangProzessor.REL_LOESCH_ANFORDERUNG)).isEmpty();
			}
		}
	}

	@DisplayName("Is loeschen anfordern allowed")
	@Nested
	class TestIsLoeschenAnfordernAllowed {

		@DisplayName("on matching status")
		@Nested
		class TestOnMatchingStatus {

			@DisplayName("with VERWALTUNG_USER role")
			@Nested
			class TestWithVerwaltungUserRole {

				@BeforeEach
				void mock() {
					when(currentUserService.hasRole(UserRole.VERWALTUNG_USER)).thenReturn(true);
				}

				@ParameterizedTest
				@EnumSource(names = { "VERWORFEN", "ABGESCHLOSSEN" })
				void shouldReturnTrue(VorgangStatus vorgangStatus) {
					var vorgang = VorgangHeaderTestFactory.createBuilder().status(vorgangStatus).build();

					var isAllowed = prozessor.isLoeschenAnfordernAllowed(vorgang);

					assertThat(isAllowed).isTrue();
				}
			}

			@DisplayName("with EINHEITLICHER_ANSPRECHPARTNER role")
			@Nested
			class TestWithEinheitlicherAnsprechpartnerRole {

				@BeforeEach
				void mock() {
					when(currentUserService.hasRole(UserRole.VERWALTUNG_USER)).thenReturn(false);
					when(currentUserService.hasRole(UserRole.EINHEITLICHER_ANSPRECHPARTNER)).thenReturn(true);
				}

				@ParameterizedTest
				@EnumSource(names = { "VERWORFEN", "ABGESCHLOSSEN" })
				void shouldReturnTrue(VorgangStatus vorgangStatus) {
					var vorgang = VorgangHeaderTestFactory.createBuilder().status(vorgangStatus).build();

					var isAllowed = prozessor.isLoeschenAnfordernAllowed(vorgang);

					assertThat(isAllowed).isTrue();
				}
			}
		}

		@ParameterizedTest
		@EnumSource(mode = Mode.EXCLUDE, names = { "VERWORFEN", "ABGESCHLOSSEN" })
		void shouldReturnFalseForNonMatchingVorgangStatus(VorgangStatus vorgangStatus) {
			var vorgang = VorgangHeaderTestFactory.createBuilder().status(vorgangStatus).build();

			var isAllowed = prozessor.isLoeschenAnfordernAllowed(vorgang);

			assertThat(isAllowed).isFalse();
		}
	}
}