/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.vorgang;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.util.List;
import java.util.Optional;
import java.util.stream.Stream;

import org.apache.commons.lang3.StringUtils;
import org.hamcrest.core.StringEndsWith;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;

import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItemService;
import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItemTestFactory;
import de.ozgcloud.alfa.common.clientattribute.ClientAttributeService;
import de.ozgcloud.alfa.common.command.CommandController;
import de.ozgcloud.alfa.common.errorhandling.ResourceNotFoundException;
import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserProfileTestFactory;
import de.ozgcloud.alfa.common.user.UserService;
import de.ozgcloud.alfa.postfach.PostfachMailController;
import de.ozgcloud.alfa.statistic.ByStatusTestFactory;
import de.ozgcloud.alfa.statistic.StatisticController;
import de.ozgcloud.alfa.statistic.StatisticTestFactory;
import de.ozgcloud.alfa.vorgang.forwarding.ForwardingController;
import de.ozgcloud.alfa.wiedervorlage.WiedervorlageTestFactory;

@AutoConfigureMockMvc
@SpringBootTest
@WithMockUser
class VorgangControllerITCase {

	@MockBean
	private VorgangRemoteService remoteService;
	@MockBean
	private CommandController commandController;
	@MockBean
	private ForwardingController forwardingController;
	@MockBean
	private PostfachMailController postfachMailController;
	@MockBean
	private ClientAttributeService clientAttributeService;
	@MockBean
	private StatisticController statisticController;
	@MockBean
	private VorgangAttachedItemService vorgangAttachedItemService;

	@MockBean
	private CurrentUserService currentUserService;
	@MockBean
	private UserService userService;

	@Autowired
	private MockMvc mockMvc;

	@BeforeEach
	void mock() {
		when(currentUserService.getUser()).thenReturn(UserProfileTestFactory.create());
		when(userService.getInternalId(any())).thenReturn(Optional.of(UserProfileTestFactory.INTERNAL_ID));
		when(statisticController.getVorgaengeStatistic()).thenReturn(StatisticTestFactory.create());
		when(vorgangAttachedItemService.findLoeschAnforderung(anyString())).thenReturn(Stream.of(VorgangAttachedItemTestFactory.create()));
	}

	@DisplayName("Vorganglist by page and limit")
	@Nested
	class TestVorgangListByPage {

		private static final String PATH = VorgangController.PATH + "?page=0&limit=100";

		private final VorgaengeHeaderResponse response = VorgaengeHeaderResponse.builder().vorgaengeHeader(List.of(
				VorgangHeaderTestFactory.createBuilder().nextFrist(WiedervorlageTestFactory.FRIST).build())).build();

		@BeforeEach
		void mockRemoteService() {
			when(remoteService.findVorgaengeHeader(any(), any())).thenReturn(response);
		}

		@Test
		void shouldReturnResult() throws Exception {
			var response = doRequest();

			response.andExpect(jsonPath("$._embedded.vorgangHeaderList[0].nextFrist").value(WiedervorlageTestFactory.FRIST_STR));
		}

		@Test
		void shouldReturnStatusOk() throws Exception {
			var response = doRequest();

			response.andExpect(status().isOk());
		}

		private ResultActions doRequest() throws Exception {
			return mockMvc.perform(get(PATH, VorgangHeaderTestFactory.ID));
		}

		@DisplayName("statistic")
		@Nested
		class TestStatisic {

			@DisplayName("by status")
			@Nested
			class TestByStatus {

				private static final String BY_STATUS_PATH = "$.statistic.byStatus.";

				@Test
				void shouldContainsNeu() throws Exception {
					var response = doRequest();

					response.andExpect(jsonPath(BY_STATUS_PATH + "neu").value(ByStatusTestFactory.NEU_COUNT));
				}

				@Test
				void shouldContainsAngenommen() throws Exception {
					var response = doRequest();

					response.andExpect(jsonPath(BY_STATUS_PATH + "angenommen").value(ByStatusTestFactory.ANGENOMMEN_COUNT));
				}

				@Test
				void shouldContainsInBearbeitung() throws Exception {
					var response = doRequest();

					response.andExpect(jsonPath(BY_STATUS_PATH + "inBearbeitung").value(ByStatusTestFactory.IN_BEARBEITUNG_COUNT));
				}

				@Test
				void shouldContainsBeschieden() throws Exception {
					var response = doRequest();

					response.andExpect(jsonPath(BY_STATUS_PATH + "beschieden").value(ByStatusTestFactory.BESCHIEDEN_COUNT));
				}

				@Test
				void shouldContainsAbgeschlossen() throws Exception {
					var response = doRequest();

					response.andExpect(jsonPath(BY_STATUS_PATH + "abgeschlossen").value(ByStatusTestFactory.ABGESCHLOSSEN_COUNT));
				}

				@Test
				void shouldContainsVerworfen() throws Exception {
					var response = doRequest();

					response.andExpect(jsonPath(BY_STATUS_PATH + "verworfen").value(ByStatusTestFactory.VERWORFEN_COUNT));
				}
			}

			@Test
			void shouldContainsWiedervorlagen() throws Exception {
				var response = doRequest();

				response.andExpect(jsonPath("$.statistic.wiedervorlagen").value(StatisticTestFactory.COUNT_WIEDERVORLAGEN));
			}
		}
	}

	@Nested
	class TestGetSingleVorgang {

		public static final String PATH = VorgangController.PATH + "/{id}";

		@Test
		void shouldReturnNotFound() throws Exception {
			when(remoteService.findVorgangWithEingang(anyString(), any())).thenThrow(new ResourceNotFoundException(Vorgang.class, StringUtils.EMPTY));

			var response = doRequest();

			response.andExpect(status().isNotFound());
		}

		@Test
		void shouldFormatDateTime() throws Exception {
			when(remoteService.findVorgangWithEingang(anyString(), any())).thenReturn(VorgangWithEingangTestFactory.create());

			var response = doRequest();

			response.andExpect(status().isOk()).andExpect(jsonPath("$.createdAt").value(VorgangHeaderTestFactory.CREATED_AT_STR));
		}

		@Test
		void shouldHaveAnnehmenLink() throws Exception {
			when(remoteService.findVorgangWithEingang(anyString(), any())).thenReturn(VorgangWithEingangTestFactory.create());

			var response = doRequest();

			response.andExpect(jsonPath("$._links.annehmen.href")
					.value("http://localhost/api/vorgangs/" + VorgangHeaderTestFactory.ID + "/relations/" + VorgangHeaderTestFactory.ID + "/"
							+ VorgangHeaderTestFactory.VERSION + "/commands"));
		}

		@Nested
		class TestLinkToAssignedUser {

			@Test
			void shouldReturnStatusOk() throws Exception {
				when(remoteService.findVorgangWithEingang(anyString(), any())).thenReturn(VorgangWithEingangTestFactory.create());

				var response = doRequest();

				response.andExpect(status().is2xxSuccessful());
			}

			@Test
			void shouldBePresentIfAssigned() throws Exception {
				when(remoteService.findVorgangWithEingang(anyString(), any())).thenReturn(VorgangWithEingangTestFactory.create());

				var response = doRequest();

				response.andExpect(jsonPath("$._links.assignedTo.href")
						.value(StringEndsWith.endsWith("/api/userProfiles/" + UserProfileTestFactory.ID)));
			}

			@Test
			void shouldNOTBePresentIfNOSTAssigned() throws Exception {
				when(remoteService.findVorgangWithEingang(anyString(), any()))
						.thenReturn(VorgangWithEingangTestFactory.createBuilder().assignedTo(null).build());

				var response = doRequest();

				response.andExpect(jsonPath("$._links.assignedTo.href").doesNotExist());
			}
		}

		private ResultActions doRequest() throws Exception {
			return mockMvc.perform(get(PATH, VorgangHeaderTestFactory.ID).with(csrf()));
		}
	}

	@Nested
	class TestClientAttribute {

		@Test
		void shouldReturnNoContent() throws Exception {
			var content = VorgangHeaderTestFactory.createHasNewPostfachNachrichtContent(false);

			var response = doRequest(content);

			response.andExpect(status().isNoContent());
		}

		@Test
		void shouldReturnUnprocessableEntity() throws Exception {
			var content = VorgangHeaderTestFactory.createHasNewPostfachNachrichtContent(true);

			var response = doRequest(content);

			response.andExpect(status().isUnprocessableEntity());
		}

		private ResultActions doRequest(String content) throws Exception {
			return mockMvc.perform(put("/api/vorgangs/{0}/hasNewPostfachNachricht", VorgangHeaderTestFactory.ID).with(csrf())
					.contentType(MediaType.APPLICATION_JSON)
					.content(content));
		}
	}
}