/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { RouterTestingModule } from '@angular/router/testing';
import { ApiRootFacade } from '@goofy-client/api-root-shared';
import { ENVIRONMENT_CONFIG } from '@goofy-client/environment-shared';
import { NavigationService } from '@goofy-client/navigation-shared';
import { createEmptyStateResource, createStateResource } from '@goofy-client/tech-shared';
import { mock, notExistsAsHtmlElement } from '@goofy-client/test-utils';
import { IconService, SpinnerComponent } from '@goofy-client/ui';
import { AuthConfig, OAuthEvent, OAuthService } from 'angular-oauth2-oidc';
import { createApiRootResource } from 'libs/api-root-shared/test/api-root';
import { BuildInfoComponent } from 'libs/navigation/src/lib/build-info/build-info.component';
import { HeaderContainerComponent } from 'libs/navigation/src/lib/header-container/header-container.component';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { setWindowLocationUrl } from 'libs/tech-shared/test/window';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { AppComponent } from './app.component';

import * as VorgangNavigationUtil from 'libs/vorgang-shared/src/lib/vorgang-navigation.util';

registerLocaleData(localeDe);

describe('AppComponent', () => {
	let component: AppComponent;
	let fixture: ComponentFixture<AppComponent>;

	const envConfig = { authServer: 'http://' };
	const apiRootFacade = mock(ApiRootFacade);
	const iconService = mock(IconService);

	const login = Promise.resolve();
	const authService = { ...mock(OAuthService), loadDiscoveryDocumentAndLogin: () => login, events: of(<OAuthEvent>{}) };

	const navigationService = mock(NavigationService);

	const buildInfo: string = getDataTestIdOf('build-info');

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			imports: [
				RouterTestingModule
			],
			declarations: [
				AppComponent,
				MockComponent(SpinnerComponent),
				MockComponent(HeaderContainerComponent),
				MockComponent(BuildInfoComponent)
			],
			providers: [
				{
					provide: ENVIRONMENT_CONFIG,
					useValue: envConfig
				},
				{
					provide: ApiRootFacade,
					useValue: apiRootFacade
				},
				{
					provide: OAuthService,
					useValue: authService
				},
				{
					provide: IconService,
					useValue: iconService
				},
				{
					provide: NavigationService,
					useValue: navigationService
				}
			]
		}).compileComponents();
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(AppComponent);
		component = fixture.componentInstance;
		fixture.detectChanges();
	});

	it('should create the app', () => {
		expect(component).toBeTruthy();
	});

	it(`should have "goofy" as title`, () => {
		expect(component.title).toEqual('goofy');
	});

	it(`should call iconService registerIcons`, () => {
		expect(iconService.registerIcons).toHaveBeenCalled();
	});

	describe('buildInfo', () => {

		it('FIXME(Von der Reihenfolge abhaengig) should NOT be rendered if api root not loaded', () => {
			// apiRootFacade.getApiRoot.mockReturnValue(of(createEmptyStateResource()));
			// apiRootFacade.getApiRoot.mockReturnValue(apiRootSubj);
			// fixture.detectChanges();
			// apiRootSubj.next(createEmptyStateResource());
			// component.ngOnInit();
			// fixture.detectChanges();
			// await login;
			// console.info('Should NOT be rendered...');

			notExistsAsHtmlElement(fixture, buildInfo);
		})

		it.skip('should be rendered', async () => {
			apiRootFacade.getApiRoot.mockReturnValue(of(createStateResource(createApiRootResource())));
			// fixture.detectChanges();
			// apiRootSubj.next(createStateResource(createApiRootResource()));
			component.ngOnInit();
			// fixture.detectChanges();
			await login;

			// console.info('Should be rendered...');

			const header = fixture.nativeElement.querySelector(buildInfo);
			expect(header).toBeInstanceOf(HTMLElement);
		})
	})

	describe('ngOnInit', () => {
		it('should call proceedWithLogin', () => {
			component.proceedWithLogin = jest.fn();

			component.ngOnInit();

			expect(component.proceedWithLogin).toHaveBeenCalled();
		})

		it('should call listenToOAuthEvents', () => {
			component.listenToOAuthEvents = jest.fn();

			component.ngOnInit();

			expect(component.listenToOAuthEvents).toHaveBeenCalled();
		})
	})

	describe('proceedWithLogin', () => {

		it('should configure oAuthService', () => {
			const configuration: AuthConfig = {};
			component.buildConfiguration = jest.fn().mockReturnValue(configuration);

			component.proceedWithLogin();

			expect(authService.configure).toHaveBeenCalledWith(configuration);
		})

		it('should build configuration', () => {
			component.buildConfiguration = jest.fn();

			component.proceedWithLogin();

			expect(component.buildConfiguration).toHaveBeenCalled();
		})

		it('should setup automatic silent refresh', () => {
			component.proceedWithLogin();

			expect(authService.setupAutomaticSilentRefresh).toHaveBeenCalled();
		})

		it('should call loadDiscoveryDocumentAndLogin', () => {
			const spy = jest.spyOn(authService, 'loadDiscoveryDocumentAndLogin');

			component.proceedWithLogin();

			expect(spy).toHaveBeenCalled();
		})
	})

	describe('listenToOAuthEvents', () => {
		it('should call consideredAsLoginEvent', () => {
			component.consideredAsLoginEvent = jest.fn();

			component.listenToOAuthEvents();

			expect(component.consideredAsLoginEvent).toHaveBeenCalled();
		})

		it('should call consideredAsPageReloadEvent', () => {
			component.consideredAsPageReloadEvent = jest.fn();

			component.listenToOAuthEvents();

			expect(component.consideredAsPageReloadEvent).toHaveBeenCalled();

		})

		it('should call loadApiRootAndNavigate at event "token_received"', () => {
			component.loadApiRootAndNavigate = jest.fn();
			authService.events = of({type: 'token_received'});

			component.listenToOAuthEvents();

			expect(component.loadApiRootAndNavigate).toHaveBeenCalled();
		})

		it('should call loadApiRootAndNavigate at event "discovery_document_loaded"', () => {
			component.loadApiRootAndNavigate = jest.fn();
			authService.events = of({type: 'discovery_document_loaded'});
			jest.spyOn(authService, 'hasValidAccessToken').mockReturnValue(true);
			jest.spyOn(authService, 'hasValidIdToken').mockReturnValue(true);

			component.listenToOAuthEvents();

			expect(component.loadApiRootAndNavigate).toHaveBeenCalled();
		})

		it('should not call loadApiRootAndNavigate at other event', () => {
			component.loadApiRootAndNavigate = jest.fn();
			authService.events = of({type: 'token_error'});

			component.listenToOAuthEvents();

			expect(component.loadApiRootAndNavigate).not.toHaveBeenCalled();
		})
	})

	describe('consideredAsLoginEvent', () => {
		it('should return true if event is "token_received"', () => {
			const event: string = 'token_received';

			const result: boolean = component.consideredAsLoginEvent(event);

			expect(result).toBeTruthy();
		})

		it('should return false if event is not "token_received"', () => {
			const event: string = 'something_else';

			const result: boolean = component.consideredAsLoginEvent(event);

			expect(result).toBeFalsy();
		})
	})

	describe('consideredAsPageReloadEvent', () => {
		it('should return true if event is "discovery_document_loaded" and tokens are valid', () => {
			component.hasValidToken = jest.fn().mockReturnValue(true);
			const event: string = 'discovery_document_loaded';

			const result: boolean = component.consideredAsPageReloadEvent(event);

			expect(result).toBeTruthy();
		})

		it('should return false if event is "discovery_document_loaded" and tokens are invalid', () => {
			component.hasValidToken = jest.fn().mockReturnValue(false);
			const event: string = 'discovery_document_loaded';

			const result: boolean = component.consideredAsPageReloadEvent(event);

			expect(result).toBeFalsy();
		})

		it('should return false if event is not "discovery_document_loaded" and tokens are valid', () => {
			component.hasValidToken = jest.fn().mockReturnValue(true);
			const event: string = 'something_else';

			const result: boolean = component.consideredAsPageReloadEvent(event);

			expect(result).toBeFalsy();
		})

		it('should return false if event is not "discovery_document_loaded" and tokens are invalid', () => {
			component.hasValidToken = jest.fn().mockReturnValue(false);
			const event: string = 'something_else';

			const result: boolean = component.consideredAsPageReloadEvent(event);

			expect(result).toBeFalsy();
		})
	})

	describe('hasValidToken', () => {
		it('should return true if both tokens are valid', () => {
			jest.spyOn(authService, 'hasValidAccessToken').mockReturnValue(true);
			jest.spyOn(authService, 'hasValidIdToken').mockReturnValue(true);

			const result: boolean = component.hasValidToken();

			expect(result).toBeTruthy();
		})

		it('should return false if access token is invalid', () => {
			jest.spyOn(authService, 'hasValidAccessToken').mockReturnValue(false);
			jest.spyOn(authService, 'hasValidIdToken').mockReturnValue(true);

			const result: boolean = component.hasValidToken();

			expect(result).toBeFalsy();
		})

		it('should return false if id token is invalid', () => {
			jest.spyOn(authService, 'hasValidAccessToken').mockReturnValue(true);
			jest.spyOn(authService, 'hasValidIdToken').mockReturnValue(false);

			const result: boolean = component.hasValidToken();

			expect(result).toBeFalsy();
		})

		it('should return false if both tokens are invalid', () => {
			jest.spyOn(authService, 'hasValidAccessToken').mockReturnValue(false);
			jest.spyOn(authService, 'hasValidIdToken').mockReturnValue(false);

			const result: boolean = component.hasValidToken();

			expect(result).toBeFalsy();
		})
	})

	describe('loadApiRootAndNavigate', () => {

		describe('apiRootResource loaded', () => {
			const apiRootStateResource = createStateResource(createApiRootResource());

			beforeEach(() => {
				apiRootFacade.getApiRoot.mockReturnValue(of(apiRootStateResource));
			})

			it('should call apiRootFacade.getApiRoot to get the apiRoot', () => {
				component.loadApiRootAndNavigate();

				expect(apiRootFacade.getApiRoot).toHaveBeenCalled();
			})

			it('should call navigationService.navigate', (done) => {
				const spy = jest.spyOn(navigationService, 'navigate');

				component.loadApiRootAndNavigate().subscribe(() => {
					expect(spy).toHaveBeenCalled();
					done();
				});
			})

			it('should call buildInitialPath', (done) => {
				component.buildInitialPath = jest.fn();

				component.loadApiRootAndNavigate().subscribe(() => {
					expect(component.buildInitialPath).toHaveBeenCalled();
					done();
				});
			})

		})

		describe('apiRoot.resource is null', () => {
			it.skip('should not call navigationService.navigate', (done) => {
				apiRootFacade.getApiRoot.mockReturnValue(of(createEmptyStateResource()));
				const spy = jest.spyOn(navigationService, 'navigate');

				component.loadApiRootAndNavigate().subscribe(() => {
					expect(spy).not.toHaveBeenCalled();
					done();
				});
			})
		})

	})

	describe('buildInitialPath', () => {

		describe('with existing window.location.pathname', () => {
			const path: string = '/vorgang/xyz';
			const url: string = 'http://localhost' + path;

			beforeEach(() => {
				setWindowLocationUrl(url);
			})

			it('should not call buildPathSegmentsFromLocalStorage', () => {
				const spy = jest.spyOn(VorgangNavigationUtil, 'buildPathSegmentsFromLocalStorage');

				component.buildInitialPath();

				expect(spy).not.toHaveBeenCalled();
			})


			it('should return window.location.pathname', () => {
				const redirectPath: string = component.buildInitialPath();

				expect(redirectPath).toBe(path);
			})
		})

		describe('with empty window.location.pathname', () => {
			const url: string = 'http://localhost/';

			beforeEach(() => {
				setWindowLocationUrl(url);
			})

			it('should call buildPathSegmentsFromLocalStorage', () => {
				const spy = jest.spyOn(VorgangNavigationUtil, 'buildPathSegmentsFromLocalStorage');

				component.buildInitialPath();

				expect(spy).toHaveBeenCalled();
			})

			it('should return path segments from localStorage', () => {
				const pathSegments: string[] = ['alle', 'angenommen'];
				jest.spyOn(VorgangNavigationUtil, 'buildPathSegmentsFromLocalStorage').mockReturnValue(pathSegments);

				const redirectUri: string = component.buildInitialPath();

				expect(redirectUri).toBe('/alle/angenommen');
			})
		})

	})
});
