/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { CommandOrder, CommandResource, CommandService } from '@goofy-client/command-shared';
import { NavigationService } from '@goofy-client/navigation-shared';
import { StateResource, createEmptyStateResource, createErrorStateResource, createStateResource } from '@goofy-client/tech-shared';
import { Mock, mock, useFromMock } from '@goofy-client/test-utils';
import { CreateForwardCommand, ForwardRequest, VorgangResource, VorgangService, VorgangWithEingangLinkRel, VorgangWithEingangResource } from '@goofy-client/vorgang-shared';
import { cold, hot } from 'jest-marbles';
import { CommandLinkRel } from 'libs/command-shared/src/lib/command.linkrel';
import { createCommandResource } from 'libs/command-shared/test/command';
import { createVorgangForwardRequest, createVorgangResource, createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { of } from 'rxjs';
import { createApiError } from '../../../tech-shared/test/error';
import { createForwardingListResource, createForwardingResource } from '../../test/forwarding';
import { ForwardingLinkRel } from './forwarding.linkrel';
import { ForwardingListResource, ForwardingResource } from './forwarding.model';
import { ForwardingRepository } from './forwarding.repository';
import { ForwardingService } from './forwarding.service';

describe('ForwardingService', () => {
	let service: ForwardingService;
	let commandService: Mock<CommandService>;
	let vorgangService: Mock<VorgangService>;
	let navigationService: Mock<NavigationService>;
	let forwardingRepository: Mock<ForwardingRepository>;

	const forwardRequest: ForwardRequest = createVorgangForwardRequest();
	const forwading: ForwardingResource = createForwardingResource();

	beforeEach(() => {
		commandService = mock(CommandService);
		vorgangService = mock(VorgangService);
		navigationService = mock(NavigationService);
		forwardingRepository = mock(ForwardingRepository);

		service = new ForwardingService(useFromMock(commandService), useFromMock(vorgangService), useFromMock(navigationService), useFromMock(forwardingRepository));
	})

	it('should create', () => {
		expect(service).toBeTruthy();
	});

	describe('forward', () => {

		const vorgang: VorgangResource = createVorgangResource();
		const commandStateResource: StateResource<CommandResource> = createStateResource(createCommandResource());

		beforeEach(() => {
			commandService.createCommand.mockReturnValue(of(commandStateResource));
			vorgangService.getPendingForwardCommand(of(commandStateResource));
		})

		it('should call commandService', () => {
			const command: CreateForwardCommand = { order: CommandOrder.REDIRECT_VORGANG, redirectRequest: forwardRequest, body: null };

			service.forward(vorgang, forwardRequest);

			expect(commandService.createCommand).toHaveBeenCalledWith(vorgang, VorgangWithEingangLinkRel.FORWARD, command);
		})

		it('should call reloadCurrentVorgang', () => {
			service.reloadCurrentVorgang = jest.fn();

			service.forward(vorgang, forwardRequest);

			expect(service.reloadCurrentVorgang).toHaveBeenCalled();
		})

		it('should call setPendingForwardSingleCommandLoading', () => {
			service.forward(vorgang, forwardRequest);

			expect(vorgangService.setPendingForwardSingleCommandLoading).toHaveBeenCalled();
		})

		it('should call setPendingForwardSingleCommand', () => {
			service.forward(vorgang, forwardRequest);

			expect(vorgangService.setPendingForwardSingleCommand).toHaveBeenCalled();
		})
	})

	describe('reloadCurrentVorgang', () => {

		it('should call vorgang service', () => {
			service.reloadCurrentVorgang(createEmptyStateResource());

			expect(vorgangService.reloadCurrentVorgang).toHaveBeenCalled();
		})

		it('should NOT call vorgang service on error', () => {
			service.reloadCurrentVorgang(createErrorStateResource(createApiError()));

			expect(vorgangService.reloadCurrentVorgang).not.toHaveBeenCalled();
		})
	})

	describe('pollPendingForwardCommand', () => {

		const commandReturnValue: StateResource<CommandResource> = createStateResource({ ...createCommandResource() });
		const commandResourceWithUpdateLink: CommandResource = { ...createCommandResource([CommandLinkRel.UPDATE]) };

		beforeEach(() => {
			commandService.pollCommand.mockReturnValue(of(commandReturnValue));
			service.reloadCurrentVorgang = jest.fn();
		})

		it('should call commandService on pending forward command', () => {
			service.pollPendingForwardCommand(createStateResource({ ...createCommandResource() }));

			expect(commandService.pollCommand).not.toHaveBeenCalled();
		})

		it('should not call commandService', () => {
			service.pollPendingForwardCommand(createStateResource(commandResourceWithUpdateLink));

			expect(commandService.pollCommand).toHaveBeenCalledWith(commandResourceWithUpdateLink);
		})

		it('should call vorgangservice on command is done', () => {
			const commandServiceReturnValue = createStateResource(createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]));
			commandService.pollCommand.mockReturnValue(of(commandServiceReturnValue));

			service.pollPendingForwardCommand(createStateResource(commandResourceWithUpdateLink));

			expect(vorgangService.setPendingForwardSingleCommand).toHaveBeenCalledWith(commandServiceReturnValue);

		})

		it('should call not vorgangservice on pending command', () => {
			const commandServiceReturnValue = createStateResource(createCommandResource());
			commandService.pollCommand.mockReturnValue(of(commandServiceReturnValue));

			service.pollPendingForwardCommand(createStateResource(commandResourceWithUpdateLink));

			expect(vorgangService.setPendingForwardSingleCommand).not.toHaveBeenCalled();
		})

		it('should call reloadCurrentVorgang on command is done', () => {
			const command: CommandResource = createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]);
			const commandServiceReturnValue = createStateResource(command);
			commandService.pollCommand.mockReturnValue(of(commandServiceReturnValue));

			service.pollPendingForwardCommand(createStateResource(commandResourceWithUpdateLink));

			expect(service.reloadCurrentVorgang).toHaveBeenCalledWith(commandServiceReturnValue);
		})
	})

	describe('onNavigation', () => {

		beforeEach(() => {
			service.unsubscribe = jest.fn();
		})

		describe('on leaving vorgang-detail page', () => {

			it('should call unsubscribe on subscriptions', () => {
				service.onNavigation({});

				expect(service.unsubscribe).toHaveBeenCalled();
			})
		})
	})

	describe('getPendingForwardCommand', () => {

		beforeEach(() => {
			service.listenToNavigation = jest.fn();
		})

		it('should call listenToNavigation', () => {
			vorgangService.getPendingForwardCommand.mockReturnValue(of({}));

			service.getPendingForwardCommand();

			expect(service.listenToNavigation).toHaveBeenCalled();
		})

		it('should call vorgang service', () => {
			vorgangService.getPendingForwardCommand.mockReturnValue(of({}));

			service.getPendingForwardCommand();

			expect(vorgangService.getPendingForwardCommand).toHaveBeenCalled();
		})
	})

	describe('getForwardings', () => {

		const vorgang: VorgangWithEingangResource = createVorgangWithEingangResource();
		const listResource: ForwardingListResource = createForwardingListResource();

		beforeEach(() => {
			forwardingRepository.getForwardings.mockReturnValue(hot('-a', { a: listResource }));
		})

		it('should return value', () => {
			const result = service.getForwardings(vorgang);

			expect(result).toBeObservable(cold('ab', { a: createEmptyStateResource(true), b: createStateResource(listResource) }));
		})

		it('should call repository', () => {
			service.getForwardings(vorgang);

			expect(forwardingRepository.getForwardings).toHaveBeenCalledWith(vorgang);
		})
	})

	describe('markAs', () => {

		const stateResource: StateResource<ForwardingResource> = createStateResource(forwading);

		beforeEach(() => {
			commandService.createCommand.mockReturnValue(hot('-a', { a: stateResource }));
		})

		describe('createMarkAsCommand', () => {

			it('should return value', () => {
				const result = service.createMarkAsCommand(forwading, ForwardingLinkRel.MARK_AS_SUCCESS, { order: CommandOrder.FORWARD_SUCCESSFULL, body: null });

				expect(result).toBeObservable(cold('ab', { a: createEmptyStateResource(true), b: stateResource }));
			})
		})

		describe('markAsSuccess', () => {

			it('should call command service', () => {
				service.markAsSuccess(forwading);

				expect(commandService.createCommand).toHaveBeenCalledWith(forwading, ForwardingLinkRel.MARK_AS_SUCCESS, { order: CommandOrder.FORWARD_SUCCESSFULL, body: null });
			})

			it('should update list on valid response', () => {
				commandService.createCommand.mockReturnValue(of(stateResource));

				service.updateForwardingList = jest.fn();

				service.markAsSuccess(forwading).subscribe();

				expect(service.updateForwardingList).toHaveBeenCalled();
			})
		})

		describe('markAsFail', () => {

			it('should call command service', () => {
				service.markAsFail(forwading);

				expect(commandService.createCommand).toHaveBeenCalledWith(forwading, ForwardingLinkRel.MARK_AS_FAIL, { order: CommandOrder.FORWARD_FAILED, body: null });
			})

			it('should update vorgang on valid response', () => {
				commandService.createCommand.mockReturnValue(of(createStateResource(createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]))));

				service.markAsFail(forwading).subscribe();

				expect(vorgangService.reloadCurrentVorgang).toHaveBeenCalled();
			})
		})
	})

	describe('update forwarding list', () => {


		beforeEach(() => {
			service.updateList = jest.fn();
		})

		it('should do nothing if link not exits', () => {
			const command: CommandResource = createCommandResource();

			service.updateForwardingList(createStateResource(command));

			expect(service.updateList).not.toHaveBeenCalled();
		})

		it('should call update list', () => {
			const command: CommandResource = createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]);

			service.updateForwardingList(createStateResource(command));

			expect(service.updateList).toHaveBeenCalledWith(command);
		})
	})

	describe('update list', () => {

		const command: CommandResource = createCommandResource();
		const list: ForwardingListResource = createForwardingListResource();

		beforeEach(() => {
			commandService.getEffectedResource.mockReturnValue(of(list));
			service.setList = jest.fn();
		})

		it('should call command service', () => {
			service.updateList(command);

			expect(commandService.getEffectedResource).toHaveBeenCalledWith(command);
		})

		it('should set fowardling list', () => {
			service.updateList(command);

			expect(service.setList).toHaveBeenCalledWith(list);
		})
	})
})