/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BinaryFileService } from '@goofy-client/binary-file-shared';
import { CommandOrder, CommandResource, CommandService } from '@goofy-client/command-shared';
import { NavigationService } from '@goofy-client/navigation-shared';
import { createEmptyStateResource, createStateResource, StateResource } from '@goofy-client/tech-shared';
import { mock, Mock, useFromMock } from '@goofy-client/test-utils';
import { VorgangService, VorgangWithEingangResource } from '@goofy-client/vorgang-shared';
import { cold, hot } from 'jest-marbles';
import { CommandLinkRel } from 'libs/command-shared/src/lib/command.linkrel';
import { createCommandResource } from 'libs/command-shared/test/command';
import { createKommentar, createKommentarListResource, createKommentarResource } from 'libs/kommentar-shared/test/kommentar';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { of } from 'rxjs';
import { KommentarLinkRel, KommentarListLinkRel } from './kommentar.linkrel';
import { CreateKommentarCommand, Kommentar, KommentarListResource, KommentarResource } from './kommentar.model';
import { KommentarRepository } from './kommentar.repository';
import { KommentarService } from './kommentar.service';

describe('KommentarService', () => {
	let service: KommentarService;
	let repository: Mock<KommentarRepository>;
	let commandService: Mock<CommandService>;
	let navigationService: Mock<NavigationService>;
	let vorgangService: Mock<VorgangService>;
	let binaryFileService: Mock<BinaryFileService>;

	const vorgang: VorgangWithEingangResource = createVorgangWithEingangResource();
	const kommentarList: KommentarListResource = createKommentarListResource();
	const kommentar: Kommentar = createKommentar();
	const kommentarResource: KommentarResource = createKommentarResource();
	const command: CommandResource = createCommandResource();
	const commandStateResource: StateResource<CommandResource> = createStateResource(command);

	beforeEach(() => {
		repository = mock(KommentarRepository);
		commandService = mock(CommandService);
		navigationService = mock(NavigationService);
		vorgangService = mock(VorgangService);
		binaryFileService = mock(BinaryFileService);

		navigationService.urlChanged.mockReturnValue(of({}));

		service = new KommentarService(
			useFromMock(repository),
			useFromMock(commandService),
			useFromMock(navigationService),
			useFromMock(vorgangService),
			useFromMock(binaryFileService));
	})

	describe('getKommentareByVorgang', () => {

		beforeEach(() => {
			repository.findKommentare.mockReturnValue(of(kommentarList));

			service.setListLoadingTrue = jest.fn();
			service.setKommentarList = jest.fn();
		})

		it('should set loading to true', () => {
			service.getKommentareByVorgang(vorgang);

			expect(service.setListLoadingTrue).toHaveBeenCalled();
		})

		it('should call repository', () => {
			service.getKommentareByVorgang(vorgang).subscribe();

			expect(repository.findKommentare).toHaveBeenCalledWith(vorgang);
		})

		it('should set loaded resource', () => {
			service.getKommentareByVorgang(vorgang);

			expect(service.setKommentarList).toHaveBeenCalledWith(kommentarList);
		})
	})

	describe('createKommentar', () => {

		describe('init and next value', () => {

			beforeEach(() => {
				commandService.createCommand.mockReturnValue(hot('-a', { a: commandStateResource }));
			})

			it('should be return', () => {
				const result = service.createKommentar(kommentar);

				expect(result).toBeObservable(cold('ab', { a: createEmptyStateResource(true), b: commandStateResource }));
			})
		})

		describe('further process', () => {

			beforeEach(() => {
				commandService.createCommand.mockReturnValue(of(commandStateResource));
			})

			it('should call command service', () => {
				service.createKommentar(kommentar);

				expect(commandService.createCommand).toHaveBeenCalledWith(service.kommentarList$.value.resource, KommentarListLinkRel.CREATE_KOMMENTAR, service.createCreateKommentarCommand(kommentar));
			})

			it('should call getEffectedResource', () => {
				service.afterCreateOrEditKommentar = jest.fn();

				service.createKommentar(kommentar).subscribe();

				expect(service.afterCreateOrEditKommentar).toHaveBeenCalledWith(commandStateResource);
			})

			it('should call createCreateKommentarCommand', () => {
				service.createCreateKommentarCommand = jest.fn();

				service.createKommentar(kommentar);

				expect(service.createCreateKommentarCommand).toHaveBeenCalledWith(kommentar);
			})

			it('should create CreateKommentarCommand', () => {
				var command: CreateKommentarCommand = service.createCreateKommentarCommand(kommentar);

				expect(command).toEqual({ order: CommandOrder.CREATE_KOMMENTAR, kommentar, body: null });
			})
		})
	})

	describe('edit kommentar', () => {

		describe('init and next value', () => {

			beforeEach(() => {
				commandService.createCommand.mockReturnValue(hot('-a', { a: commandStateResource }));
			})

			it('should be return', () => {
				const result = service.editKommentar(kommentarResource, kommentar);

				expect(result).toBeObservable(cold('ab', { a: createEmptyStateResource(true), b: commandStateResource }));
			})
		})

		describe('further process', () => {

			beforeEach(() => {
				commandService.createCommand.mockReturnValue(of(commandStateResource));
			})

			it('should call command service', () => {
				service.editKommentar(kommentarResource, kommentar);

				expect(commandService.createCommand).toHaveBeenCalledWith(kommentarResource, KommentarLinkRel.EDIT, service.createEditKommentarCommand(kommentar));
			})

			it('should call getEffectedResource', () => {
				service.afterCreateOrEditKommentar = jest.fn();

				service.editKommentar(kommentarResource, kommentar).subscribe();

				expect(service.afterCreateOrEditKommentar).toHaveBeenCalledWith(commandStateResource);
			})

			it('should call createEditKommentarCommand', () => {
				service.createEditKommentarCommand = jest.fn();

				service.editKommentar(kommentarResource, kommentar);

				expect(service.createEditKommentarCommand).toHaveBeenCalledWith(kommentar);
			})

			it('should create CreateKommentarCommand', () => {
				var command: CreateKommentarCommand = service.createEditKommentarCommand(kommentar);

				expect(command).toEqual({ order: CommandOrder.EDIT_KOMMENTAR, kommentar, body: null });
			})
		})
	})

	describe('afterCreateOrEditKommentar', () => {

		describe('on command is done', () => {

			const kommentarList: KommentarListResource = createKommentarListResource();

			beforeEach(() => {
				commandService.getEffectedResource.mockReturnValue(of(kommentarList));
			})

			it('should hide formular', () => {
				service.hideFormular = jest.fn();

				service.afterCreateOrEditKommentar({ ...commandStateResource, resource: createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]) });

				expect(service.hideFormular).toHaveBeenCalled();
			})

			it('should reload vorgang', () => {
				service.afterCreateOrEditKommentar({ ...commandStateResource, resource: createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]) });

				expect(vorgangService.reloadCurrentVorgang).toHaveBeenCalled();
			})

			it('should set kommentar list on reload', () => {
				service.setKommentarListReload = jest.fn();

				service.afterCreateOrEditKommentar({ ...commandStateResource, resource: createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]) });

				expect(service.setKommentarListReload).toHaveBeenCalled();
			})
		})

		describe('on command is pending', () => {

			it('should do nothing', () => {
				service.afterCreateOrEditKommentar({ ...commandStateResource, resource: createCommandResource([CommandLinkRel.UPDATE]) });

				expect(commandService.getEffectedResource).not.toHaveBeenCalled();
			})
		})
	})

	describe('onNavigation', () => {

		describe('to vorgang list', () => {

			it('should set reload flag of kommentar list', () => {
				service.kommentarList$.next(createEmptyStateResource());

				service.onNavigation({});

				expect(service.kommentarList$.value.reload).toEqual(true);
			})
		})

		describe('to vorgang detail page', () => {

			it('should set kommentar list to reload', () => {
				service.kommentarList$.next(createEmptyStateResource());

				service.onNavigation({ 'vorgangWithEingangUrl': 'X' });

				expect(service.kommentarList$.value.reload).toEqual(true);
			})
		});
	})

	describe('attachments', () => {
		it('should be loaded if link avaible', () => {
			const kommentarResource = createKommentarResource([KommentarLinkRel.ATTACHMENTS]);
			service.getAttachments(kommentarResource);

			expect(binaryFileService.getFiles).toHaveBeenCalledWith(
				kommentarResource,
				KommentarLinkRel.ATTACHMENTS
			);
		});

		it('should not be loaded if no link available', () => {
			service.getAttachments(kommentarResource);

			expect(binaryFileService.getFiles).not.toHaveBeenCalled();
		})
	});

	describe('canCreateNewKommentar', () => {
		it('should create new Kommentare', () => {
			const canCreateNewKommentar$ = cold('a', {a: true});

			const observable = service.canCreateNewKommentar(createKommentarListResource([KommentarListLinkRel.CREATE_KOMMENTAR]));

			expect(observable).toBeObservable(canCreateNewKommentar$);
		});

		it('should not create new Kommentare if formular visible', () => {
			const canCreateNewKommentar$ = cold('a', {a: false});
			service.formularVisibility$.next(true);

			const observable = service.canCreateNewKommentar(createKommentarListResource([KommentarListLinkRel.CREATE_KOMMENTAR]));

			expect(observable).toBeObservable(canCreateNewKommentar$);
		});

		it('should not create new Kommentare if link not exists', () => {
			const canCreateNewKommentar$ = cold('a', {a: false});
			service.formularVisibility$.next(false);

			const observable = service.canCreateNewKommentar(createKommentarListResource());

			expect(observable).toBeObservable(canCreateNewKommentar$);
		});
	});
})