import { CommandOrder, CommandResource } from '@goofy-client/command-shared';
import { ApiError, createEmptyStateResource, createErrorStateResource, createStateResource } from '@goofy-client/tech-shared';
import { VorgangWithEingangResource } from '@goofy-client/vorgang-shared';
import { Action } from '@ngrx/store';
import { TypedAction } from '@ngrx/store/src/models';
import { createCommand, createCommandResource } from 'libs/command-shared/test/command';
import { createLoeschAnforderungResource } from 'libs/loesch-anforderung-shared/test/loesch-anforderung';
import { createApiError } from 'libs/tech-shared/test/error';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { LoeschAnforderungLinkRel } from '../loesch-anforderung.linkrel';
import { LoeschAnforderungResource } from '../loesch-anforderung.model';
import { createRouteData } from './../../../../navigation-shared/test/navigation-test-factory';
import { LoeschAnforderungState, initialState, reducer } from './loesch-anforderung.reducer';

import * as CommandActions from '../../../../command-shared/src/lib/+state/command.actions';
import * as NavigationActions from '../../../../navigation-shared/src/lib/+state/navigation.actions';
import * as VorgangActions from '../../../../vorgang-shared/src/lib/+state/vorgang.actions';
import * as LoeschAnforderungActions from './loesch-anforderung.actions';

describe('LoeschAnforderung Reducer', () => {

	describe('unknown action', () => {

		it('should return current state', () => {
			const action = {} as Action;

			const result = reducer(initialState, action);

			expect(result).toBe(initialState);
		})
	})

	describe('LoeschAnforderung', () => {

		describe('on "loadLoeschAnforderungSuccess" action', () => {

			it('should set loeschenAnfordernResource', () => {
				const loeschAnforderung: LoeschAnforderungResource = createLoeschAnforderungResource();
				const action = LoeschAnforderungActions.loadLoeschAnforderungSuccess({ loeschAnforderung });

				const state: LoeschAnforderungState = reducer(initialState, action);

				expect(state.loeschAnforderung).toEqual(createStateResource(loeschAnforderung));
			})
		})

		describe('on "loadLoeschAnforderungFailure" action', () => {

			it('should set loeschenAnfordernResource as errorStateResource', () => {
				const apiError: ApiError = createApiError();
				const action = LoeschAnforderungActions.loadLoeschAnforderungFailure({ apiError });

				const state: LoeschAnforderungState = reducer(initialState, action);

				expect(state.loeschAnforderung).toEqual(createErrorStateResource(apiError));
			})
		})
	})

	describe('VorgangZumLoeschenMarkieren', () => {

		describe('on "createCommand" action', () => {

			it('should set vorgangLoeschenCommand loading', () => {
				const action = CommandActions.createCommand({
					command: { ...createCommand(), order: CommandOrder.VORGANG_ZUM_LOESCHEN_MARKIEREN },
					resource: createVorgangWithEingangResource(),
					linkRel: LoeschAnforderungLinkRel.EXECUTE_LOESCHEN
				});

				const state: LoeschAnforderungState = reducer(initialState, action);

				expect(state.loeschenAnfordernCommand.loading).toBeTruthy();
			})
		})

		describe('on "createCommandSuccess" action', () => {

			it('should set vorgangLoeschenCommand as StateResource', () => {
				const command: CommandResource = { ...createCommandResource(), order: CommandOrder.VORGANG_ZUM_LOESCHEN_MARKIEREN };
				const action = CommandActions.createCommandSuccess({ command });

				const state: LoeschAnforderungState = reducer(initialState, action);

				expect(state.loeschenAnfordernCommand).toEqual(createStateResource(command));
			})
		})

		describe('on "revokeCommandSuccess" action', () => {

			it('should clear loeschAnforderung', () => {
				const command: CommandResource = { ...createCommandResource(), order: CommandOrder.VORGANG_ZUM_LOESCHEN_MARKIEREN };
				const action = CommandActions.revokeCommandSuccess({ command });

				const state: LoeschAnforderungState = reducer({ ...initialState, loeschAnforderung: createStateResource(createLoeschAnforderungResource()) }, action);

				expect(state.loeschAnforderung).toEqual(createEmptyStateResource());
			})
		})
	})

	describe('VorgangLoeschen', () => {

		describe('on "createCommand" action', () => {

			it('should set vorgangLoeschenCommand loading', () => {
				const action = CommandActions.createCommand({
					command: { ...createCommand(), order: CommandOrder.VORGANG_LOESCHEN },
					resource: createVorgangWithEingangResource(),
					linkRel: LoeschAnforderungLinkRel.EXECUTE_LOESCHEN
				});

				const state: LoeschAnforderungState = reducer(initialState, action);

				expect(state.vorgangLoeschenCommand.loading).toBeTruthy();
			})
		})

		describe('on "createCommandSuccess" action', () => {

			it('should set vorgangLoeschenCommand as StateResource', () => {
				const command: CommandResource = { ...createCommandResource(), order: CommandOrder.VORGANG_LOESCHEN };
				const action = CommandActions.createCommandSuccess({ command });

				const state: LoeschAnforderungState = reducer(initialState, action);

				expect(state.vorgangLoeschenCommand).toEqual(createStateResource(command));
			})
		})
	})

	describe('LoeschAnforderungZuruecknehmen', () => {

		describe('on "createCommand" action', () => {

			it('should set loeschAnforderungZuruecknehmenCommand loading', () => {
				const action = CommandActions.createCommand({
					command: { ...createCommand(), order: CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN },
					resource: createVorgangWithEingangResource(),
					linkRel: LoeschAnforderungLinkRel.ZURUECKNEHMEN
				});

				const state: LoeschAnforderungState = reducer(initialState, action);

				expect(state.loeschAnforderungZuruecknehmenCommand.loading).toBeTruthy();
			})
		})

		describe('on "createCommandSuccess" action', () => {

			it('should set loeschAnforderungZuruecknehmenCommand as StateResource', () => {
				const command: CommandResource = { ...createCommandResource(), order: CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN };
				const action = CommandActions.createCommandSuccess({ command });

				const state: LoeschAnforderungState = reducer(initialState, action);

				expect(state.loeschAnforderungZuruecknehmenCommand).toEqual(createStateResource(command));
			})

			it('should clear LoeschAnforderung state', () => {
				const command: CommandResource = { ...createCommandResource(), order: CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN };
				const action = CommandActions.createCommandSuccess({ command });

				const state: LoeschAnforderungState = reducer({
					...initialState,
					loeschAnforderung: createStateResource(createLoeschAnforderungResource())
				}, action);

				expect(state.loeschAnforderung).toEqual(createEmptyStateResource());
			})
		})

		describe('on "createCommandFailure" action', () => {

			it('should set loeschAnforderungZuruecknehmenCommand as StateResource', () => {
				const command: CommandResource = { ...createCommandResource(), order: CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN };
				const error: ApiError = createApiError();
				const action = CommandActions.createCommandFailure({ command, error });

				const state: LoeschAnforderungState = reducer(initialState, action);

				expect(state.loeschAnforderungZuruecknehmenCommand).toEqual(createErrorStateResource(error));
			})
		})

		describe('on "revokeCommandSuccess" action', () => {

			it('should reload loeschAnforderung', () => {
				const command: CommandResource = { ...createCommandResource(), order: CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN };
				const action = CommandActions.revokeCommandSuccess({ command });

				const state: LoeschAnforderungState = reducer({ ...initialState, loeschAnforderung: createStateResource(createLoeschAnforderungResource()) }, action);

				expect(state.loeschAnforderung.reload).toBeTruthy();
			})
		})
	})

	describe('Vorgang', () => {

		const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource();
		const action: TypedAction<string> = VorgangActions.loadVorgangWithEingangSuccess({ vorgangWithEingang });

		describe('loadVorgangWithEingangSuccess', () => {

			it('should clear loeschenAnfordernCommand', () => {
				const filledState: LoeschAnforderungState = { ...initialState, loeschenAnfordernCommand: createStateResource(createCommandResource()) };
				const state: LoeschAnforderungState = reducer(filledState, action);

				expect(state.loeschenAnfordernCommand).toEqual(createEmptyStateResource());
			})

			it('should clear loeschAnforderung', () => {
				const filledState: LoeschAnforderungState = { ...initialState, loeschAnforderung: createStateResource(createLoeschAnforderungResource()) };
				const state: LoeschAnforderungState = reducer(filledState, action);

				expect(state.loeschAnforderung).toEqual(createEmptyStateResource());
			})

			it('should clear vorgangLoeschenCommand', () => {
				const filledState: LoeschAnforderungState = { ...initialState, vorgangLoeschenCommand: createStateResource(createCommandResource()) };
				const state: LoeschAnforderungState = reducer(filledState, action);

				expect(state.vorgangLoeschenCommand).toEqual(createEmptyStateResource());
			})

			it('should clear loeschAnforderungZuruecknehmenCommand', () => {
				const filledState: LoeschAnforderungState = { ...initialState, loeschAnforderungZuruecknehmenCommand: createStateResource(createCommandResource()) };
				const state: LoeschAnforderungState = reducer(filledState, action);

				expect(state.loeschAnforderungZuruecknehmenCommand).toEqual(createEmptyStateResource());
			})
		})
	})

	describe('Navigation', () => {

		it('should clear loeschenAnfordernCommand', () => {
			const action: TypedAction<string> = NavigationActions.updateCurrentRouteData({ routeData: createRouteData() });

			const filledState: LoeschAnforderungState = { ...initialState, loeschenAnfordernCommand: createStateResource(createCommandResource()) };
			const state: LoeschAnforderungState = reducer(filledState, action);

			expect(state.loeschenAnfordernCommand).toEqual(createEmptyStateResource());
		})

		it('should clear loeschAnforderung', () => {
			const action: TypedAction<string> = NavigationActions.updateCurrentRouteData({ routeData: createRouteData() });

			const filledState: LoeschAnforderungState = { ...initialState, loeschAnforderung: createStateResource(createLoeschAnforderungResource()) };
			const state: LoeschAnforderungState = reducer(filledState, action);

			expect(state.loeschAnforderung).toEqual(createEmptyStateResource());
		})

		it('should clear vorgangLoeschenCommand', () => {
			const action: TypedAction<string> = NavigationActions.updateCurrentRouteData({ routeData: createRouteData() });

			const filledState: LoeschAnforderungState = { ...initialState, vorgangLoeschenCommand: createStateResource(createCommandResource()) };
			const state: LoeschAnforderungState = reducer(filledState, action);

			expect(state.vorgangLoeschenCommand).toEqual(createEmptyStateResource());
		})

		it('should clear loeschAnforderungZuruecknehmenCommand', () => {
			const action: TypedAction<string> = NavigationActions.updateCurrentRouteData({ routeData: createRouteData() });

			const filledState: LoeschAnforderungState = { ...initialState, loeschAnforderungZuruecknehmenCommand: createStateResource(createCommandResource()) };
			const state: LoeschAnforderungState = reducer(filledState, action);

			expect(state.loeschAnforderungZuruecknehmenCommand).toEqual(createEmptyStateResource());
		})
	})
})