/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatIcon } from '@angular/material/icon';
import { ON_PAGE, PostfachMailLinkRel, PostfachMailResource } from '@goofy-client/postfach-shared';
import { HasLinkPipe, StateResource, createStateResource } from '@goofy-client/tech-shared';
import { existsAsHtmlElement, getElementFromFixture, getMockComponent, notExistsAsHtmlElement } from '@goofy-client/test-utils';
import { UserProfileInPostfachMailContainerComponent } from '@goofy-client/user-profile';
import { VorgangWithEingangResource } from '@goofy-client/vorgang-shared';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { createPostfachMailResource } from '../../../../../../../postfach-shared/test/postfach';
import { PostfachMailDateComponent } from '../postfach-mail-date/postfach-mail-date.component';
import { OutgoingMailErrorContainerComponent } from './outgoing-mail-error-container/outgoing-mail-error-container.component';
import { OutgoingMailComponent } from './outgoing-mail.component';
import { PostfachNachrichtEditButtonContainerComponent } from './postfach-nachricht-edit-button-container/postfach-nachricht-edit-button-container.component';

describe('OutgoingMailComponent', () => {
	let component: OutgoingMailComponent;
	let fixture: ComponentFixture<OutgoingMailComponent>;

	const attachmentIcon: string = getDataTestIdOf('postfach-nachricht-attachment-icon');
	const postfachEditButton: string = getDataTestIdOf('postfach-nachricht-edit-button-container');
	const postfachOutgoingNachrichtDate: string = getDataTestIdOf('postfach-outgoing-nachricht-date');

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				OutgoingMailComponent,
				HasLinkPipe,
				MatIcon,
				MockComponent(OutgoingMailErrorContainerComponent),
				MockComponent(PostfachMailDateComponent),
				MockComponent(UserProfileInPostfachMailContainerComponent),
				MockComponent(PostfachMailDateComponent),
				MockComponent(PostfachNachrichtEditButtonContainerComponent)
			],
			providers: [
				{
					provide: ON_PAGE,
					useValue: undefined
				}
			]
		}).compileComponents();
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(OutgoingMailComponent);
		component = fixture.componentInstance;
		component.postfachMail = createPostfachMailResource();
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('attachments', () => {

		it('should only show attachment icon on detail', () => {
			component.onPage = false;
			component.postfachMail = createPostfachMailResource([PostfachMailLinkRel.ATTACHMENTS]);
			fixture.detectChanges();

			const element = getElementFromFixture(fixture, attachmentIcon);

			expect(element).toBeInstanceOf(HTMLElement);
		})
	})

	describe('edit button container component', () => {

		const postfachNachricht: PostfachMailResource = createPostfachMailResource([PostfachMailLinkRel.EDIT]);

		describe('on existing edit link', () => {

			beforeEach(() => {
				component.postfachMail = postfachNachricht;
			})

			it('should be visbile', () => {
				fixture.detectChanges();

				existsAsHtmlElement(fixture, postfachEditButton);
			})

			it('should be called with postfachNachricht', () => {
				fixture.detectChanges();

				const editButtonComponent: PostfachNachrichtEditButtonContainerComponent = getMockComponent(fixture, PostfachNachrichtEditButtonContainerComponent);
				expect(editButtonComponent.postfachNachricht).toBe(postfachNachricht);
			})

			it('should be called with vorgangStateResource', () => {
				const vorgangStateResource: StateResource<VorgangWithEingangResource> = createStateResource(createVorgangWithEingangResource());
				component.vorgangStateResource = vorgangStateResource;

				fixture.detectChanges();

				const editButtonComponent: PostfachNachrichtEditButtonContainerComponent = getMockComponent(fixture, PostfachNachrichtEditButtonContainerComponent);
				expect(editButtonComponent.vorgangStateResource).toBe(vorgangStateResource);
			})
		})

		describe('on missing edit link', () => {

			const postfachNachricht: PostfachMailResource = createPostfachMailResource();

			beforeEach(() => {
				component.postfachMail = postfachNachricht;
			})

			it('should not be visible', () => {
				fixture.detectChanges();

				notExistsAsHtmlElement(fixture, postfachEditButton);
			})
		})

		describe('in detail context', () => {

			const postfachNachricht: PostfachMailResource = createPostfachMailResource([PostfachMailLinkRel.EDIT]);

			beforeEach(() => {
				component.onPage = true;
				component.postfachMail = postfachNachricht;
			})

			it('should not be visible', () => {
				fixture.detectChanges();

				notExistsAsHtmlElement(fixture, postfachEditButton);
			})
		})
	})

	describe('date', () => {

		it('should be visible if exists', () => {
			component.postfachMail = createPostfachMailResource();

			fixture.detectChanges();

			existsAsHtmlElement(fixture, postfachOutgoingNachrichtDate);
		})

		it('should not be visible if not exists', () => {
			component.postfachMail = { ...createPostfachMailResource(), sentAt: null };

			fixture.detectChanges();

			notExistsAsHtmlElement(fixture, postfachOutgoingNachrichtDate);
		})
	})
});
