/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { HttpResponse, HttpStatusCode } from '@angular/common/http';
import { buildBlobWithFileName, isChangingDataRequest, isConnectionTimeout, isForbidden, isServerError, isServiceUnavailable, isUnauthorized, isUnprocessableEntity } from './http.util';
import { BlobWithFileName, HttpMethod } from './tech.model';

describe('http util', () => {

	const SERVER_ERROR_CODES: HttpStatusCode[] = [500, 501, 505, 506, 507, 508, 510, 511];
	const CONNECTION_TIMEOUT_CODES: HttpStatusCode[] = [502, 503, 504];
	const CLIENT_ERROR: HttpStatusCode[] = [400, 401, 402, 403, 404, 422];

	describe('isForbidden', () => {

		it.each(CLIENT_ERROR.slice(HttpStatusCode.Forbidden).concat(SERVER_ERROR_CODES))
			('should be false on %i', (httpStatusCode: HttpStatusCode) => {
				const result: boolean = isForbidden(httpStatusCode);

				expect(result).toBeFalsy();
			});

		it('should be true on 403', () => {
			const result: boolean = isForbidden(HttpStatusCode.Forbidden);

			expect(result).toBeTruthy();
		})
	})

	describe('isUnauthorized', () => {

		it.each(CLIENT_ERROR.slice(HttpStatusCode.Unauthorized).concat(SERVER_ERROR_CODES))
			('should be false on %i', (httpStatusCode: HttpStatusCode) => {
				const result: boolean = isUnauthorized(httpStatusCode);

				expect(result).toBeFalsy();
			});

		it('should be true on 401', () => {
			const result: boolean = isUnauthorized(HttpStatusCode.Unauthorized);

			expect(result).toBeTruthy();
		});
	})

	describe('isServerError', () => {

		it.each(SERVER_ERROR_CODES)
			('should be true on %i', (httpStatusCode: HttpStatusCode) => {
				const result: boolean = isServerError(httpStatusCode);

				expect(result).toBeTruthy();
			});

		it.each(CLIENT_ERROR.concat(CONNECTION_TIMEOUT_CODES))
			('should be false on %i', () => {
				const result: boolean = isServerError(HttpStatusCode.Forbidden);

				expect(result).toBeFalsy();
			})
	})

	describe('isUnprocessableEntity', () => {

		it.each(CLIENT_ERROR.slice(HttpStatusCode.UnprocessableEntity).concat(SERVER_ERROR_CODES).concat(SERVER_ERROR_CODES))
			('should be false on %i', (httpStatusCode: HttpStatusCode) => {
				const result: boolean = isUnprocessableEntity(httpStatusCode);

				expect(result).toBeFalsy();
			});

		it('should be true on %i', () => {
			const result: boolean = isUnprocessableEntity(HttpStatusCode.UnprocessableEntity);

			expect(result).toBeTruthy();
		})
	})

	describe('isChangingDataRequest', () => {

		const CHANGING_DATA_REQUEST_METHOD: string[] = [HttpMethod.POST, HttpMethod.PUT, HttpMethod.PATCH, HttpMethod.DELETE];

		it.each(CHANGING_DATA_REQUEST_METHOD)
			('should return true on %i', (method: string) => {
				const result: boolean = isChangingDataRequest(<any>{ method });

				expect(result).toBeTruthy();
			})

		it('should return false on %i', () => {
			const result: boolean = isChangingDataRequest(<any>{ method: HttpMethod.GET });

			expect(result).toBeFalsy();
		})
	})

	describe('isConnectionTimeout', () => {

		it.each(CONNECTION_TIMEOUT_CODES)
			('should return true on %i', (statusCode: number) => {
				const result: boolean = isConnectionTimeout(statusCode);

				expect(result).toBeTruthy();
			})

		it.each(SERVER_ERROR_CODES)
			('should return false on %i', (statusCode: number) => {
				const result: boolean = isConnectionTimeout(statusCode);

				expect(result).toBeFalsy();
			})
	})

	describe('isServiceUnavailable', () => {

		it('should return true on %i', () => {
			const result: boolean = isServiceUnavailable(HttpStatusCode.ServiceUnavailable);

			expect(result).toBeTruthy();
		})

		it.each(SERVER_ERROR_CODES.concat(CONNECTION_TIMEOUT_CODES.slice(HttpStatusCode.ServiceUnavailable)))
			('should return false on %i', (statusCode: number) => {
				const result: boolean = isServiceUnavailable(statusCode);

				expect(result).toBeFalsy();
			})
	})

	describe('buildBlobWithFileName', () => {

		const body: Blob = new Blob();
		const fileName: string = '927ee047-0123-4c36-911c-52e984accc87_Abgabe.Abgabe.0401.xdomea';
		const headerString: string = 'attachment; filename=' + fileName
		const httpResponse: HttpResponse<Blob> = <any>{ body, headers: { get: () => headerString }};

		it('should have blob', () => {
			const blobWithFileName: BlobWithFileName = buildBlobWithFileName(httpResponse);

			expect(blobWithFileName.blob).toEqual(body);
		})

		it('should have fileName', () => {
			const blobWithFileName: BlobWithFileName = buildBlobWithFileName(httpResponse);

			expect(blobWithFileName.fileName).toEqual(fileName);
		})
	})
})