/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { HttpErrorResponse, HttpHandler, HttpRequest, HttpStatusCode } from '@angular/common/http';
import { HttpErrorHandler } from '@goofy-client/tech-shared';
import { mock, useFromMock } from '@goofy-client/test-utils';
import { DialogService } from '@goofy-client/ui';
import { RetryInTimeDialog } from '../ui/dialog/retry-in-time.dialog';
import { HttpConnectionTimeoutInterceptor } from './http-connection-timeout.interceptor';

describe('HttpConnectionTimeoutInterceptor', () => {

	let interceptor: HttpConnectionTimeoutInterceptor
	const dialogService = mock(DialogService);
	const errorHandler = mock(HttpErrorHandler);
	const tokenExtractor = <any>{ getToken: () => jest.fn };

	const request: HttpRequest<unknown> = <any>{};
	const next: HttpHandler = <any>{};
	const retryDialog = mock(RetryInTimeDialog);

	beforeAll(() => {
		interceptor = new HttpConnectionTimeoutInterceptor(useFromMock(dialogService), useFromMock(errorHandler), tokenExtractor);

		dialogService.getRetryDialog.mockReturnValue(retryDialog);
	})

	describe('handle error', () => {

		it('should initRetryDialog', () => {
			interceptor.retryDialog = null;

			executeHandleError(HttpStatusCode.ServiceUnavailable);

			expect(interceptor.retryDialog).not.toBeNull();
		})

		it('shouldCallService', () => {
			executeHandleError();

			expect(dialogService.getRetryDialog).toHaveBeenCalled();
		})

		it('should not call service if already initiated', () => {
			dialogService.getRetryDialog.mockClear();
			interceptor.doRetry = jest.fn();
			interceptor.retryDialog = <any>{ shouldRetry: () => true };

			executeHandleError();

			expect(dialogService.getRetryDialog).not.toHaveBeenCalled();
		})

		it('should do nothing on no connection timeout status', () => {
			dialogService.getRetryDialog.mockClear();

			executeHandleError(HttpStatusCode.InternalServerError);

			expect(dialogService.getRetryDialog).not.toHaveBeenCalled();
		})

		it('should do retry', () => {
			interceptor.retryDialog = <any>{ shouldRetry: () => true };
			interceptor.doRetry = jest.fn();

			executeHandleError();

			expect(interceptor.doRetry).toHaveBeenCalled();
		})

		function executeHandleError(status: number = HttpStatusCode.ServiceUnavailable) {
			interceptor.handleError(request, next, <HttpErrorResponse>{ status }, true);
		}
	})

	describe('handleErrorRetry', () => {

		describe('on connection timeout error', () => {

			const retryDialog = <any>{ shouldRetry: () => true, finish: () => jest.fn() };

			it('should do retry if condition fits', () => {
				interceptor.retryDialog = retryDialog;
				interceptor.doRetry = jest.fn();

				executeHandleErrorRetry();

				expect(interceptor.doRetry).toHaveBeenCalledWith(request, next);
			})

			it('should unset retryDialog if condition does not fit', () => {
				interceptor.retryDialog = { ...retryDialog, shouldRetry: () => false };

				executeHandleErrorRetry();

				expect(interceptor.retryDialog).toBeNull();
			})

			it('should call stop retry if condition does not fit', () => {
				interceptor.retryDialog = { ...retryDialog, shouldRetry: () => false };
				interceptor.stopRetry = jest.fn();

				executeHandleErrorRetry();

				expect(interceptor.stopRetry).toHaveBeenCalled();
			})

			function executeHandleErrorRetry(status: number = HttpStatusCode.ServiceUnavailable) {
				interceptor.handleErrorRetry(request, next, <HttpErrorResponse>{ status });
			}
		})
	})

	describe('handleResponse', () => {

		const retryDialog = <any>{};

		describe('on valid response', () => {

			const response = { type: 4, ok: true };

			it('should close all retry dialogs if response is valid', () => {
				interceptor.handleResponse(response);

				expect(dialogService.closeById).toHaveBeenCalledWith(RetryInTimeDialog.ID);
			})

			it('should unset retryDialog', () => {
				interceptor.retryDialog = retryDialog;

				interceptor.handleResponse(response);

				expect(interceptor.retryDialog).toBeNull();
			})
		})

		describe('on invalid response', () => {

			const response = { type: 0 };

			beforeEach(() => {
				dialogService.closeById.mockClear();
			})

			it('should do nothing on invalid response', () => {
				interceptor.retryDialog = retryDialog;

				interceptor.handleResponse(response);

				expect(dialogService.closeById).not.toHaveBeenCalled();
				expect(interceptor.retryDialog).toStrictEqual(retryDialog);
			})
		})

	})
})