/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { LOCALE_ID } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MAT_DATE_LOCALE } from '@angular/material/core';
import { MatIcon } from '@angular/material/icon';
import { MatIconTestingModule } from '@angular/material/icon/testing';
import { RouterTestingModule } from '@angular/router/testing';
import { ConvertForDataTestPipe, EnumToLabelPipe, HasLinkPipe, ToResourceUriPipe, convertForDataTest } from '@goofy-client/tech-shared';
import { getElementFromFixture } from '@goofy-client/test-utils';
import { PostfachIconComponent } from '@goofy-client/ui';
import { UserProfileInVorgangListItemContainerComponent } from '@goofy-client/user-profile';
import { VorgangHeaderLinkRel } from '@goofy-client/vorgang-shared';
import { AktenzeichenComponent, VorgangNummerComponent, VorgangStatusDotComponent, VorgangStatusTextComponent } from '@goofy-client/vorgang-shared-ui';
import { WiedervorlageListInVorgangListContainerComponent } from '@goofy-client/wiedervorlage';
import { getDataTestClassOf, getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MatTooltipDirective } from 'libs/ui/src/lib/ui/mattooltip/mattooltip.directive';
import { createVorgangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { VorgangCreatedAtComponent } from './vorgang-created-at/vorgang-created-at.component';
import { VorgangListItemComponent } from './vorgang-list-item.component';
import { VorgangNextFristButton } from './vorgang-next-frist-button/vorgang-next-frist-button.component';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('VorgangListItemComponent', () => {
	let component: VorgangListItemComponent;
	let fixture: ComponentFixture<VorgangListItemComponent>;

	const user: string = getDataTestIdOf('vorgang-user-icon');
	const postfachStatus: string = getDataTestClassOf('postfach-icon');

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			imports: [
				RouterTestingModule,
				MatIconTestingModule
			],
			declarations: [
				VorgangListItemComponent,
				MatIcon,
				MatTooltipDirective,
				EnumToLabelPipe,
				ToResourceUriPipe,
				HasLinkPipe,
				ConvertForDataTestPipe,
				MockComponent(AktenzeichenComponent),
				MockComponent(VorgangNummerComponent),
				MockComponent(PostfachIconComponent),
				MockComponent(VorgangStatusDotComponent),
				MockComponent(VorgangStatusTextComponent),
				MockComponent(WiedervorlageListInVorgangListContainerComponent),
				MockComponent(VorgangNextFristButton),
				MockComponent(UserProfileInVorgangListItemContainerComponent),
				MockComponent(VorgangCreatedAtComponent)
			],
			providers: [
				{ provide: LOCALE_ID, useValue: 'de' },
				{ provide: MAT_DATE_LOCALE, useValue: 'de-DE' }
			],
		})
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(VorgangListItemComponent);
		component = fixture.componentInstance;
		component.vorgang = createVorgangResource([VorgangHeaderLinkRel.VORGANG_WITH_EINGANG]);
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('bearbeiter/user-icon', () => {

		it('should be visible', () => {
			component.vorgang = createVorgangResource([VorgangHeaderLinkRel.VORGANG_WITH_EINGANG, VorgangHeaderLinkRel.ASSIGN]);
			fixture.detectChanges();

			const userElement = fixture.nativeElement.querySelector(user);

			expect(userElement).toBeInstanceOf(HTMLElement);
		})

		it('should NOT be visible', () => {
			component.vorgang = createVorgangResource([VorgangHeaderLinkRel.VORGANG_WITH_EINGANG]);
			fixture.detectChanges();

			const userElement = fixture.nativeElement.querySelector(user);

			expect(userElement).not.toBeInstanceOf(HTMLElement);
		})
	})

	describe('Aria label', () => {
		it('should contain Wiedervorlage', () => {
			component.vorgang = createVorgangResource([VorgangHeaderLinkRel.VORGANG_WITH_EINGANG, VorgangHeaderLinkRel.WIEDERVORLAGEN]);
			const listItem: string = getDataTestIdOf(`vorgang-list-item-${convertForDataTest(component.vorgang.name)}`);
			component.ngOnInit();
			fixture.detectChanges();

			const element: HTMLDivElement = fixture.nativeElement.querySelector(listItem);
			const ariaLabel: string = element.getAttribute('aria-label');

			expect(ariaLabel).toContain('Wiedervorlage');
		})

		it('should not contain Wiedervorlage if no nextFrist but LinkRel.WIEDERVORLAGEN', () => {
			component.vorgang = { ...createVorgangResource([VorgangHeaderLinkRel.VORGANG_WITH_EINGANG, VorgangHeaderLinkRel.WIEDERVORLAGEN]), nextFrist: null };
			const listItem: string = getDataTestIdOf(`vorgang-list-item-${convertForDataTest(component.vorgang.name)}`);
			component.ngOnInit();
			fixture.detectChanges();

			const element: HTMLDivElement = fixture.nativeElement.querySelector(listItem);
			const ariaLabel: string = element.getAttribute('aria-label');

			expect(ariaLabel).not.toContain('Wiedervorlage');
		})

		it('should not contain Wiedervorlage if no LinkRel.WIEDERVORLAGEN', () => {
			component.vorgang = createVorgangResource([VorgangHeaderLinkRel.VORGANG_WITH_EINGANG]);
			const listItem: string = getDataTestIdOf(`vorgang-list-item-${convertForDataTest(component.vorgang.name)}`);
			component.ngOnInit();
			fixture.detectChanges();

			const element: HTMLDivElement = fixture.nativeElement.querySelector(listItem);
			const ariaLabel: string = element.getAttribute('aria-label');

			expect(ariaLabel).not.toContain('Wiedervorlage');
		})

		it('should contain hasPostfachnachricht text if vorgang has Postfachnachricht', () => {
			component.vorgang.hasPostfachNachricht = true;
			component.vorgang.hasNewPostfachNachricht = false;
			const listItem: string = getDataTestIdOf(`vorgang-list-item-${convertForDataTest(component.vorgang.name)}`);
			component.ngOnInit();
			fixture.detectChanges();

			const element: HTMLDivElement = fixture.nativeElement.querySelector(listItem);
			const ariaLabel: string = element.getAttribute('aria-label');

			expect(ariaLabel).toContain('enthält Postfachnachrichten');
		})

		it('should contain hasNewPostfachnachricht text if vorgang has new Postfachnachricht', () => {
			component.vorgang.hasPostfachNachricht = true;
			component.vorgang.hasNewPostfachNachricht = true;
			const listItem: string = getDataTestIdOf(`vorgang-list-item-${convertForDataTest(component.vorgang.name)}`);
			component.ngOnInit();
			fixture.detectChanges();

			const element: HTMLDivElement = fixture.nativeElement.querySelector(listItem);
			const ariaLabel: string = element.getAttribute('aria-label');

			expect(ariaLabel).toContain('enthält neue Postfachnachrichten');
		})
	})

	describe('mail icon', () => {
		beforeEach(() => {
			component.vorgang = createVorgangResource([VorgangHeaderLinkRel.VORGANG_WITH_EINGANG, VorgangHeaderLinkRel.POSTFACH_MAILS]);
		})

		it('should show mail icon if Vorgang has new Postfachnachricht', () => {
			component.vorgang.hasPostfachNachricht = true;
			fixture.detectChanges();

			const statusElement = getElementFromFixture(fixture, postfachStatus);

			expect(statusElement).toBeInstanceOf(HTMLElement);
		})

		it('should not show mail icon if Vorgang has no new Postfachnachricht', () => {
			component.vorgang.hasPostfachNachricht = false;
			fixture.detectChanges();

			const statusElement = getElementFromFixture(fixture, postfachStatus);

			expect(statusElement).not.toBeInstanceOf(HTMLElement);
		})
	})
});
