/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormBuilder, FormControl, FormGroup, ReactiveFormsModule } from '@angular/forms';
import { MatButtonToggle, MatButtonToggleGroup } from '@angular/material/button-toggle';
import { MatRipple } from '@angular/material/core';
import { ApiRootLinkRel } from '@goofy-client/api-root-shared';
import { NavigationService } from '@goofy-client/navigation-shared';
import { createEmptyStateResource, createStateResource, HasLinkPipe } from '@goofy-client/tech-shared';
import { existsAsHtmlElement, mock, notExistsAsHtmlElement } from '@goofy-client/test-utils';
import { VorgangListService } from '@goofy-client/vorgang-shared';
import { createApiRootResource } from 'libs/api-root-shared/test/api-root';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { ButtonToggleGroupComponent } from 'libs/ui/src/lib/ui/button-toggle-group/button-toggle-group.component';
import { ButtonToggleComponent } from 'libs/ui/src/lib/ui/button-toggle/button-toggle.component';
import { MockComponent } from 'ng-mocks';
import { VorgangFilterMenuItemComponent } from './vorgang-filter-menu-item/vorgang-filter-menu-item.component';
import { VorgangFilterMenuComponent } from './vorgang-filter-menu.component';
import { VorgangFilterFormService } from './vorgang-filter-menu.formservice';

describe('VorgangFilterMenuComponent', () => {
	let component: VorgangFilterMenuComponent;
	let fixture: ComponentFixture<VorgangFilterMenuComponent>;

	const alleFilterToggleButton: string = getDataTestIdOf('alle-filter-toggle-button');
	const meineVorgaengeFilterToggleButton: string = getDataTestIdOf('meine-vorgaenge-filter-toggle-button');
	const unassignFilterToggleButton: string = getDataTestIdOf('unassigned-filter-toggle-button');

	const form: FormGroup = new FormBuilder().group({
		filter: new FormControl()
	});

	const vorgangFilterFormService = { ...mock(VorgangFilterFormService) , form };
	const navigationService = mock(NavigationService);
	const vorgangListService = mock(VorgangListService);

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				VorgangFilterMenuComponent,
				HasLinkPipe,
				ButtonToggleGroupComponent,
				MatButtonToggleGroup,
				MatButtonToggle,
				MatRipple,
				MockComponent(ButtonToggleGroupComponent),
				MockComponent(ButtonToggleComponent),
				MockComponent(VorgangFilterMenuItemComponent),
			],
			imports: [
				ReactiveFormsModule,
			],
			providers: [
				FormBuilder,
				{
					provide: VorgangFilterFormService,
					useValue: vorgangFilterFormService
				},
				{
					provide: NavigationService,
					useValue: navigationService
				},
				{
					provide: VorgangListService,
					useValue: vorgangListService
				}
			]
		}).compileComponents();
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(VorgangFilterMenuComponent);
		component = fixture.componentInstance;
		component.apiRootStateResource = createStateResource(createApiRootResource());
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('alle filter item', () => {

		beforeEach(() => {
			jest.spyOn(component, 'areMultipleFilterAvailable').mockReturnValue(true);
		})

		it('should show on existing links', () => {
			component.apiRootStateResource = createStateResource(createApiRootResource([ApiRootLinkRel.ALLE_VORGAENGE]));

			fixture.detectChanges();

			existsAsHtmlElement(fixture, alleFilterToggleButton);
		})

		it('should hide on missing link', () => {
			component.apiRootStateResource = createStateResource(createApiRootResource());

			fixture.detectChanges();

			notExistsAsHtmlElement(fixture, alleFilterToggleButton);
		})
	})

	describe('meine vorgaenge filter item', () => {

		beforeEach(() => {
			jest.spyOn(component, 'areMultipleFilterAvailable').mockReturnValue(true);
		})

		it('should show on existing link', () => {
			component.apiRootStateResource = createStateResource(createApiRootResource([ApiRootLinkRel.MEINE_VORGAENGE]));

			fixture.detectChanges();

			existsAsHtmlElement(fixture, meineVorgaengeFilterToggleButton);
		})

		it('should hide on missing link', () => {
			component.apiRootStateResource = createStateResource(createApiRootResource());

			fixture.detectChanges();

			notExistsAsHtmlElement(fixture, meineVorgaengeFilterToggleButton);
		})
	})

	describe('unassigned filter item', () => {

		beforeEach(() => {
			jest.spyOn(component, 'areMultipleFilterAvailable').mockReturnValue(true);
		})

		it('should show on existing link', () => {
			component.apiRootStateResource = createStateResource(createApiRootResource([ApiRootLinkRel.UNASSIGNED]));

			fixture.detectChanges();

			existsAsHtmlElement(fixture, unassignFilterToggleButton);
		})

		it('should hide on missing link', () => {
			component.apiRootStateResource = createStateResource(createApiRootResource());

			fixture.detectChanges();

			notExistsAsHtmlElement(fixture, unassignFilterToggleButton);
		})
	})

	describe('areMultipleFilterAvailable', () => {

		it('should return true if more than one link', () => {
			component.apiRootStateResource = createStateResource(createApiRootResource([ApiRootLinkRel.ALLE_VORGAENGE, ApiRootLinkRel.MEINE_VORGAENGE]));

			const result: boolean = component.areMultipleFilterAvailable();

			expect(result).toBeTruthy();
		})

		it('should return false if only one link', () => {
			component.apiRootStateResource = createStateResource(createApiRootResource([ApiRootLinkRel.ALLE_VORGAENGE]));

			const result: boolean = component.areMultipleFilterAvailable();

			expect(result).toBeFalsy();
		})

		it('should return false if no link', () => {
			component.apiRootStateResource = createEmptyStateResource();

			const result: boolean = component.areMultipleFilterAvailable();

			expect(result).toBeFalsy();
		})
	})
});
